@@ -28,7 +28,6 @@
 #include "config.h"
 #endif
 
-#include "cogl.h"
 #include "cogl-debug.h"
 #include "cogl-internal.h"
 #include "cogl-util.h"
@@ -43,6 +42,8 @@
 #include "cogl-journal-private.h"
 #include "cogl-pipeline-opengl-private.h"
 #include "cogl-atlas.h"
+#include "cogl1-context.h"
+#include "cogl-sub-texture.h"
 
 #include <stdlib.h>
 
@@ -809,6 +810,12 @@ _cogl_atlas_texture_remove_reorganize_callback (GHookFunc callback,
     g_hook_destroy_link (&ctx->atlas_reorganize_callbacks, hook);
 }
 
+static CoglTextureType
+_cogl_atlas_texture_get_type (CoglTexture *tex)
+{
+  return COGL_TEXTURE_TYPE_2D;
+}
+
 static const CoglTextureVtable
 cogl_atlas_texture_vtable =
   {
@@ -829,5 +836,6 @@ cogl_atlas_texture_vtable =
     _cogl_atlas_texture_get_gl_format,
     _cogl_atlas_texture_get_width,
     _cogl_atlas_texture_get_height,
+    _cogl_atlas_texture_get_type,
     NULL /* is_foreign */
   };
