@@ -25,7 +25,6 @@
 #include "config.h"
 #endif
 
-#include "cogl.h"
 #include "cogl-debug.h"
 #include "cogl-internal.h"
 #include "cogl-context-private.h"
@@ -41,6 +40,9 @@
 #include "cogl-winsys-private.h"
 #include "cogl-pipeline-state-private.h"
 #include "cogl-matrix-private.h"
+#include "cogl-primitive-private.h"
+#include "cogl-offscreen.h"
+#include "cogl1-context.h"
 
 #ifndef GL_FRAMEBUFFER
 #define GL_FRAMEBUFFER		0x8D40
@@ -109,11 +111,15 @@ typedef struct _CoglFramebufferStackEntry
   CoglFramebuffer *read_buffer;
 } CoglFramebufferStackEntry;
 
-extern GQuark _cogl_object_onscreen_get_type (void);
+extern CoglObjectClass _cogl_onscreen_class;
+
+static CoglUserDataKey wire_pipeline_key;
 
 static void _cogl_offscreen_free (CoglOffscreen *offscreen);
 
-COGL_OBJECT_DEFINE (Offscreen, offscreen);
+COGL_OBJECT_DEFINE_WITH_CODE (Offscreen, offscreen,
+                              _cogl_offscreen_class.virt_unref =
+                              _cogl_framebuffer_unref);
 COGL_OBJECT_DEFINE_DEPRECATED_REF_COUNTING (offscreen);
 
 /* XXX:
@@ -136,8 +142,8 @@ _cogl_is_framebuffer (void *object)
   if (obj == NULL)
     return FALSE;
 
-  return obj->klass->type == _cogl_object_onscreen_get_type ()
-    || obj->klass->type == _cogl_object_offscreen_get_type ();
+  return (obj->klass == &_cogl_onscreen_class ||
+          obj->klass == &_cogl_offscreen_class);
 }
 
 void
@@ -698,7 +704,7 @@ _cogl_framebuffer_init_bits (CoglFramebuffer *framebuffer)
   framebuffer->dirty_bitmasks = FALSE;
 }
 
-CoglHandle
+CoglOffscreen *
 _cogl_offscreen_new_to_texture_full (CoglTexture *texture,
                                      CoglOffscreenFlags create_flags,
                                      unsigned int level)
@@ -768,7 +774,7 @@ _cogl_offscreen_new_to_texture_full (CoglTexture *texture,
   return ret;
 }
 
-CoglHandle
+CoglOffscreen *
 cogl_offscreen_new_to_texture (CoglTexture *texture)
 {
   return _cogl_offscreen_new_to_texture_full (texture, 0, 0);
@@ -1167,19 +1173,7 @@ _cogl_set_framebuffers (CoglFramebuffer *draw_buffer,
 
   if (current_draw_buffer != draw_buffer ||
       current_read_buffer != read_buffer)
-    {
-      /* XXX: eventually we want to remove this implicit journal flush
-       * so we can log into the journal beyond framebuffer changes to
-       * support batching scenes that depend on the results of
-       * mid-scene renders to textures. Current will be NULL when the
-       * framebuffer stack is first created so we need to guard
-       * against that here */
-      if (current_draw_buffer)
-        _cogl_framebuffer_flush_journal (current_draw_buffer);
-      if (current_read_buffer)
-        _cogl_framebuffer_flush_journal (current_read_buffer);
-      _cogl_set_framebuffers_real (draw_buffer, read_buffer);
-    }
+    _cogl_set_framebuffers_real (draw_buffer, read_buffer);
 }
 
 void
@@ -1292,19 +1286,10 @@ cogl_pop_framebuffer (void)
 
   if (to_pop->draw_buffer != to_restore->draw_buffer ||
       to_pop->read_buffer != to_restore->read_buffer)
-    {
-      /* XXX: eventually we want to remove this implicit journal flush
-       * so we can log into the journal beyond framebuffer changes to
-       * support batching scenes that depend on the results of
-       * mid-scene renders to textures. */
-      _cogl_framebuffer_flush_journal (to_pop->draw_buffer);
-      _cogl_framebuffer_flush_journal (to_pop->read_buffer);
-
-      notify_buffers_changed (to_pop->draw_buffer,
-                              to_restore->draw_buffer,
-                              to_pop->read_buffer,
-                              to_restore->read_buffer);
-    }
+    notify_buffers_changed (to_pop->draw_buffer,
+                            to_restore->draw_buffer,
+                            to_pop->read_buffer,
+                            to_restore->read_buffer);
 
   cogl_object_unref (to_pop->draw_buffer);
   cogl_object_unref (to_pop->read_buffer);
@@ -2407,3 +2392,563 @@ _cogl_framebuffer_restore_clip_stack (CoglFramebuffer *framebuffer)
     framebuffer->context->current_draw_buffer_changes |=
       COGL_FRAMEBUFFER_STATE_CLIP;
 }
+
+void
+_cogl_framebuffer_unref (CoglFramebuffer *framebuffer)
+{
+  /* The journal holds a reference to the framebuffer whenever it is
+     non-empty. Therefore if the journal is non-empty and we will have
+     exactly one reference then we know the journal is the only thing
+     keeping the framebuffer alive. In that case we want to flush the
+     journal and let the framebuffer die. It is fine at this point if
+     flushing the journal causes something else to take a reference to
+     it and it comes back to life */
+  if (framebuffer->journal->entries->len > 0)
+    {
+      unsigned int ref_count = ((CoglObject *) framebuffer)->ref_count;
+
+      /* There should be at least two references - the one we are
+         about to drop and the one held by the journal */
+      if (ref_count < 2)
+        g_warning ("Inconsistent ref count on a framebuffer with journal "
+                   "entries.");
+
+      if (ref_count == 2)
+        _cogl_framebuffer_flush_journal (framebuffer);
+    }
+
+  /* Chain-up */
+  _cogl_object_default_unref (framebuffer);
+}
+
+#ifdef COGL_ENABLE_DEBUG
+static int
+get_index (void *indices,
+           CoglIndicesType type,
+           int _index)
+{
+  if (!indices)
+    return _index;
+
+  switch (type)
+    {
+    case COGL_INDICES_TYPE_UNSIGNED_BYTE:
+      return ((guint8 *)indices)[_index];
+    case COGL_INDICES_TYPE_UNSIGNED_SHORT:
+      return ((guint16 *)indices)[_index];
+    case COGL_INDICES_TYPE_UNSIGNED_INT:
+      return ((guint32 *)indices)[_index];
+    }
+
+  g_return_val_if_reached (0);
+}
+
+static void
+add_line (guint32 *line_indices,
+          int base,
+          void *user_indices,
+          CoglIndicesType user_indices_type,
+          int index0,
+          int index1,
+          int *pos)
+{
+  index0 = get_index (user_indices, user_indices_type, index0);
+  index1 = get_index (user_indices, user_indices_type, index1);
+
+  line_indices[(*pos)++] = base + index0;
+  line_indices[(*pos)++] = base + index1;
+}
+
+static int
+get_line_count (CoglVerticesMode mode, int n_vertices)
+{
+  if (mode == COGL_VERTICES_MODE_TRIANGLES &&
+      (n_vertices % 3) == 0)
+    {
+      return n_vertices;
+    }
+  else if (mode == COGL_VERTICES_MODE_TRIANGLE_FAN &&
+           n_vertices >= 3)
+    {
+      return 2 * n_vertices - 3;
+    }
+  else if (mode == COGL_VERTICES_MODE_TRIANGLE_STRIP &&
+           n_vertices >= 3)
+    {
+      return 2 * n_vertices - 3;
+    }
+    /* In the journal we are a bit sneaky and actually use GL_QUADS
+     * which isn't actually a valid CoglVerticesMode! */
+#ifdef HAVE_COGL_GL
+  else if (mode == GL_QUADS && (n_vertices % 4) == 0)
+    {
+      return n_vertices;
+    }
+#endif
+
+  g_return_val_if_reached (0);
+}
+
+static CoglIndices *
+get_wire_line_indices (CoglContext *ctx,
+                       CoglVerticesMode mode,
+                       int first_vertex,
+                       int n_vertices_in,
+                       CoglIndices *user_indices,
+                       int *n_indices)
+{
+  int n_lines;
+  guint32 *line_indices;
+  CoglIndexBuffer *index_buffer;
+  void *indices;
+  CoglIndicesType indices_type;
+  int base = first_vertex;
+  int pos;
+  int i;
+  CoglIndices *ret;
+
+  if (user_indices)
+    {
+      index_buffer = cogl_indices_get_buffer (user_indices);
+      indices = cogl_buffer_map (COGL_BUFFER (index_buffer),
+                                 COGL_BUFFER_ACCESS_READ, 0);
+      indices_type = cogl_indices_get_type (user_indices);
+    }
+  else
+    {
+      index_buffer = NULL;
+      indices = NULL;
+      indices_type = COGL_INDICES_TYPE_UNSIGNED_BYTE;
+    }
+
+  n_lines = get_line_count (mode, n_vertices_in);
+
+  /* Note: we are using COGL_INDICES_TYPE_UNSIGNED_INT so 4 bytes per index. */
+  line_indices = g_malloc (4 * n_lines * 2);
+
+  pos = 0;
+
+  if (mode == COGL_VERTICES_MODE_TRIANGLES &&
+      (n_vertices_in % 3) == 0)
+    {
+      for (i = 0; i < n_vertices_in; i += 3)
+        {
+          add_line (line_indices, base, indices, indices_type, i,   i+1, &pos);
+          add_line (line_indices, base, indices, indices_type, i+1, i+2, &pos);
+          add_line (line_indices, base, indices, indices_type, i+2, i,   &pos);
+        }
+    }
+  else if (mode == COGL_VERTICES_MODE_TRIANGLE_FAN &&
+           n_vertices_in >= 3)
+    {
+      add_line (line_indices, base, indices, indices_type, 0, 1, &pos);
+      add_line (line_indices, base, indices, indices_type, 1, 2, &pos);
+      add_line (line_indices, base, indices, indices_type, 0, 2, &pos);
+
+      for (i = 3; i < n_vertices_in; i++)
+        {
+          add_line (line_indices, base, indices, indices_type, i - 1, i, &pos);
+          add_line (line_indices, base, indices, indices_type, 0,     i, &pos);
+        }
+    }
+  else if (mode == COGL_VERTICES_MODE_TRIANGLE_STRIP &&
+           n_vertices_in >= 3)
+    {
+      add_line (line_indices, base, indices, indices_type, 0, 1, &pos);
+      add_line (line_indices, base, indices, indices_type, 1, 2, &pos);
+      add_line (line_indices, base, indices, indices_type, 0, 2, &pos);
+
+      for (i = 3; i < n_vertices_in; i++)
+        {
+          add_line (line_indices, base, indices, indices_type, i - 1, i, &pos);
+          add_line (line_indices, base, indices, indices_type, i - 2, i, &pos);
+        }
+    }
+    /* In the journal we are a bit sneaky and actually use GL_QUADS
+     * which isn't actually a valid CoglVerticesMode! */
+#ifdef HAVE_COGL_GL
+  else if (mode == GL_QUADS && (n_vertices_in % 4) == 0)
+    {
+      for (i = 0; i < n_vertices_in; i += 4)
+        {
+          add_line (line_indices,
+                    base, indices, indices_type, i,     i + 1, &pos);
+          add_line (line_indices,
+                    base, indices, indices_type, i + 1, i + 2, &pos);
+          add_line (line_indices,
+                    base, indices, indices_type, i + 2, i + 3, &pos);
+          add_line (line_indices,
+                    base, indices, indices_type, i + 3, i,     &pos);
+        }
+    }
+#endif
+
+  if (user_indices)
+    cogl_buffer_unmap (COGL_BUFFER (index_buffer));
+
+  *n_indices = n_lines * 2;
+
+  ret = cogl_indices_new (ctx,
+                          COGL_INDICES_TYPE_UNSIGNED_INT,
+                          line_indices,
+                          *n_indices);
+
+  g_free (line_indices);
+
+  return ret;
+}
+
+static gboolean
+remove_layer_cb (CoglPipeline *pipeline,
+                 int layer_index,
+                 void *user_data)
+{
+  cogl_pipeline_remove_layer (pipeline, layer_index);
+  return TRUE;
+}
+
+static void
+pipeline_destroyed_cb (CoglPipeline *weak_pipeline, void *user_data)
+{
+  CoglPipeline *original_pipeline = user_data;
+
+  /* XXX: I think we probably need to provide a custom unref function for
+   * CoglPipeline because it's possible that we will reach this callback
+   * because original_pipeline is being freed which means cogl_object_unref
+   * will have already freed any associated user data.
+   *
+   * Setting more user data here will *probably* succeed but that may allocate
+   * a new user-data array which could be leaked.
+   *
+   * Potentially we could have a _cogl_object_free_user_data function so
+   * that a custom unref function could be written that can destroy weak
+   * pipeline children before removing user data.
+   */
+  cogl_object_set_user_data (COGL_OBJECT (original_pipeline),
+                             &wire_pipeline_key, NULL, NULL);
+
+  cogl_object_unref (weak_pipeline);
+}
+
+static void
+draw_wireframe (CoglContext *ctx,
+                CoglFramebuffer *framebuffer,
+                CoglPipeline *pipeline,
+                CoglVerticesMode mode,
+                int first_vertex,
+                int n_vertices,
+                CoglAttribute **attributes,
+                int n_attributes,
+                CoglIndices *indices)
+{
+  CoglIndices *wire_indices;
+  CoglPipeline *wire_pipeline;
+  int n_indices;
+
+  wire_indices = get_wire_line_indices (ctx,
+                                        mode,
+                                        first_vertex,
+                                        n_vertices,
+                                        indices,
+                                        &n_indices);
+
+  wire_pipeline = cogl_object_get_user_data (COGL_OBJECT (pipeline),
+                                             &wire_pipeline_key);
+
+  if (!wire_pipeline)
+    {
+      wire_pipeline =
+        _cogl_pipeline_weak_copy (pipeline, pipeline_destroyed_cb, NULL);
+
+      cogl_object_set_user_data (COGL_OBJECT (pipeline),
+                                 &wire_pipeline_key, wire_pipeline,
+                                 NULL);
+
+      /* If we have glsl then the pipeline may have an associated
+       * vertex program and since we'd like to see the results of the
+       * vertex program in the wireframe we just add a final clobber
+       * of the wire color leaving the rest of the state untouched. */
+      if (cogl_has_feature (framebuffer->context, COGL_FEATURE_ID_GLSL))
+        {
+          CoglSnippet *snippet = cogl_snippet_new (COGL_SNIPPET_HOOK_FRAGMENT,
+                                                   NULL,
+                                                   NULL);
+          cogl_snippet_set_replace (snippet,
+                                    "cogl_color_out = "
+                                      "vec4 (0.0, 1.0, 0.0, 1.0);\n");
+
+          cogl_pipeline_add_snippet (wire_pipeline, snippet);
+          cogl_object_unref (snippet);
+        }
+      else
+        {
+          cogl_pipeline_foreach_layer (wire_pipeline, remove_layer_cb, NULL);
+          cogl_pipeline_set_color4f (wire_pipeline, 0, 1, 0, 1);
+        }
+    }
+
+  /* temporarily disable the wireframe to avoid recursion! */
+  COGL_DEBUG_CLEAR_FLAG (COGL_DEBUG_WIREFRAME);
+  _cogl_framebuffer_draw_indexed_attributes (
+                                           framebuffer,
+                                           wire_pipeline,
+                                           COGL_VERTICES_MODE_LINES,
+                                           0,
+                                           n_indices,
+                                           wire_indices,
+                                           attributes,
+                                           n_attributes,
+                                           COGL_DRAW_SKIP_JOURNAL_FLUSH |
+                                           COGL_DRAW_SKIP_PIPELINE_VALIDATION |
+                                           COGL_DRAW_SKIP_FRAMEBUFFER_FLUSH |
+                                           COGL_DRAW_SKIP_LEGACY_STATE);
+  COGL_DEBUG_SET_FLAG (COGL_DEBUG_WIREFRAME);
+
+  cogl_object_unref (wire_indices);
+}
+#endif
+
+/* This can be called directly by the CoglJournal to draw attributes
+ * skipping the implicit journal flush, the framebuffer flush and
+ * pipeline validation. */
+void
+_cogl_framebuffer_draw_attributes (CoglFramebuffer *framebuffer,
+                                   CoglPipeline *pipeline,
+                                   CoglVerticesMode mode,
+                                   int first_vertex,
+                                   int n_vertices,
+                                   CoglAttribute **attributes,
+                                   int n_attributes,
+                                   CoglDrawFlags flags)
+{
+#ifdef COGL_ENABLE_DEBUG
+  if (G_UNLIKELY (COGL_DEBUG_ENABLED (COGL_DEBUG_WIREFRAME)))
+    draw_wireframe (framebuffer->context,
+                    framebuffer, pipeline,
+                    mode, first_vertex, n_vertices,
+                    attributes, n_attributes, NULL);
+  else
+#endif
+    {
+      _cogl_flush_attributes_state (framebuffer, pipeline, flags,
+                                    attributes, n_attributes);
+
+      GE (framebuffer->context,
+          glDrawArrays ((GLenum)mode, first_vertex, n_vertices));
+    }
+}
+
+void
+cogl_framebuffer_draw_attributes (CoglFramebuffer *framebuffer,
+                                  CoglPipeline *pipeline,
+                                  CoglVerticesMode mode,
+                                  int first_vertex,
+                                  int n_vertices,
+                                  CoglAttribute **attributes,
+                                  int n_attributes)
+{
+  _cogl_framebuffer_draw_attributes (framebuffer,
+                                     pipeline,
+                                     mode,
+                                     first_vertex,
+                                     n_vertices,
+                                     attributes, n_attributes,
+                                     COGL_DRAW_SKIP_LEGACY_STATE);
+}
+
+void
+cogl_framebuffer_vdraw_attributes (CoglFramebuffer *framebuffer,
+                                   CoglPipeline *pipeline,
+                                   CoglVerticesMode mode,
+                                   int first_vertex,
+                                   int n_vertices,
+                                   ...)
+{
+  va_list ap;
+  int n_attributes;
+  CoglAttribute *attribute;
+  CoglAttribute **attributes;
+  int i;
+
+  va_start (ap, n_vertices);
+  for (n_attributes = 0; va_arg (ap, CoglAttribute *); n_attributes++)
+    ;
+  va_end (ap);
+
+  attributes = g_alloca (sizeof (CoglAttribute *) * n_attributes);
+
+  va_start (ap, n_vertices);
+  for (i = 0; (attribute = va_arg (ap, CoglAttribute *)); i++)
+    attributes[i] = attribute;
+  va_end (ap);
+
+  _cogl_framebuffer_draw_attributes (framebuffer,
+                                     pipeline,
+                                     mode, first_vertex, n_vertices,
+                                     attributes, n_attributes,
+                                     COGL_DRAW_SKIP_LEGACY_STATE);
+}
+
+static size_t
+sizeof_index_type (CoglIndicesType type)
+{
+  switch (type)
+    {
+    case COGL_INDICES_TYPE_UNSIGNED_BYTE:
+      return 1;
+    case COGL_INDICES_TYPE_UNSIGNED_SHORT:
+      return 2;
+    case COGL_INDICES_TYPE_UNSIGNED_INT:
+      return 4;
+    }
+  g_return_val_if_reached (0);
+}
+
+void
+_cogl_framebuffer_draw_indexed_attributes (CoglFramebuffer *framebuffer,
+                                           CoglPipeline *pipeline,
+                                           CoglVerticesMode mode,
+                                           int first_vertex,
+                                           int n_vertices,
+                                           CoglIndices *indices,
+                                           CoglAttribute **attributes,
+                                           int n_attributes,
+                                           CoglDrawFlags flags)
+{
+#ifdef COGL_ENABLE_DEBUG
+  if (G_UNLIKELY (COGL_DEBUG_ENABLED (COGL_DEBUG_WIREFRAME)))
+    draw_wireframe (framebuffer->context,
+                    framebuffer, pipeline,
+                    mode, first_vertex, n_vertices,
+                    attributes, n_attributes, indices);
+  else
+#endif
+    {
+      CoglBuffer *buffer;
+      guint8 *base;
+      size_t buffer_offset;
+      size_t index_size;
+      GLenum indices_gl_type = 0;
+
+      _cogl_flush_attributes_state (framebuffer, pipeline, flags,
+                                    attributes, n_attributes);
+
+      buffer = COGL_BUFFER (cogl_indices_get_buffer (indices));
+      base = _cogl_buffer_bind (buffer, COGL_BUFFER_BIND_TARGET_INDEX_BUFFER);
+      buffer_offset = cogl_indices_get_offset (indices);
+      index_size = sizeof_index_type (cogl_indices_get_type (indices));
+
+      switch (cogl_indices_get_type (indices))
+        {
+        case COGL_INDICES_TYPE_UNSIGNED_BYTE:
+          indices_gl_type = GL_UNSIGNED_BYTE;
+          break;
+        case COGL_INDICES_TYPE_UNSIGNED_SHORT:
+          indices_gl_type = GL_UNSIGNED_SHORT;
+          break;
+        case COGL_INDICES_TYPE_UNSIGNED_INT:
+          indices_gl_type = GL_UNSIGNED_INT;
+          break;
+        }
+
+      GE (framebuffer->context,
+          glDrawElements ((GLenum)mode,
+                          n_vertices,
+                          indices_gl_type,
+                          base + buffer_offset + index_size * first_vertex));
+
+      _cogl_buffer_unbind (buffer);
+    }
+}
+
+void
+cogl_framebuffer_draw_indexed_attributes (CoglFramebuffer *framebuffer,
+                                          CoglPipeline *pipeline,
+                                          CoglVerticesMode mode,
+                                          int first_vertex,
+                                          int n_vertices,
+                                          CoglIndices *indices,
+                                          CoglAttribute **attributes,
+                                          int n_attributes)
+{
+  _cogl_framebuffer_draw_indexed_attributes (framebuffer,
+                                             pipeline,
+                                             mode, first_vertex,
+                                             n_vertices, indices,
+                                             attributes, n_attributes,
+                                             COGL_DRAW_SKIP_LEGACY_STATE);
+}
+
+void
+cogl_vdraw_indexed_attributes (CoglFramebuffer *framebuffer,
+                               CoglPipeline *pipeline,
+                               CoglVerticesMode mode,
+                               int first_vertex,
+                               int n_vertices,
+                               CoglIndices *indices,
+                               ...)
+{
+  va_list ap;
+  int n_attributes;
+  CoglAttribute **attributes;
+  int i;
+  CoglAttribute *attribute;
+
+  va_start (ap, indices);
+  for (n_attributes = 0; va_arg (ap, CoglAttribute *); n_attributes++)
+    ;
+  va_end (ap);
+
+  attributes = g_alloca (sizeof (CoglAttribute *) * n_attributes);
+
+  va_start (ap, indices);
+  for (i = 0; (attribute = va_arg (ap, CoglAttribute *)); i++)
+    attributes[i] = attribute;
+  va_end (ap);
+
+  _cogl_framebuffer_draw_indexed_attributes (framebuffer,
+                                             pipeline,
+                                             mode,
+                                             first_vertex,
+                                             n_vertices,
+                                             indices,
+                                             attributes,
+                                             n_attributes,
+                                             COGL_DRAW_SKIP_LEGACY_STATE);
+}
+
+void
+_cogl_framebuffer_draw_primitive (CoglFramebuffer *framebuffer,
+                                  CoglPipeline *pipeline,
+                                  CoglPrimitive *primitive,
+                                  CoglDrawFlags flags)
+{
+  if (primitive->indices)
+    _cogl_framebuffer_draw_indexed_attributes (framebuffer,
+                                               pipeline,
+                                               primitive->mode,
+                                               primitive->first_vertex,
+                                               primitive->n_vertices,
+                                               primitive->indices,
+                                               primitive->attributes,
+                                               primitive->n_attributes,
+                                               flags);
+  else
+    _cogl_framebuffer_draw_attributes (framebuffer,
+                                       pipeline,
+                                       primitive->mode,
+                                       primitive->first_vertex,
+                                       primitive->n_vertices,
+                                       primitive->attributes,
+                                       primitive->n_attributes,
+                                       flags);
+}
+
+void
+cogl_framebuffer_draw_primitive (CoglFramebuffer *framebuffer,
+                                 CoglPipeline *pipeline,
+                                 CoglPrimitive *primitive)
+{
+  _cogl_framebuffer_draw_primitive (framebuffer, pipeline, primitive,
+                                    COGL_DRAW_SKIP_LEGACY_STATE);
+}
