@@ -24,43 +24,12 @@
 #ifndef __COGL_INTERNAL_H
 #define __COGL_INTERNAL_H
 
-#include "cogl.h"
-#include "cogl-matrix-stack.h"
 #include "cogl-bitmask.h"
 
 #ifdef COGL_HAS_XLIB_SUPPORT
 #include <X11/Xutil.h>
 #endif
 
-typedef enum
-{
-  COGL_FRONT_WINDING_CLOCKWISE,
-  COGL_FRONT_WINDING_COUNTER_CLOCKWISE
-} CoglFrontWinding;
-
-typedef enum {
-  COGL_BOXED_NONE,
-  COGL_BOXED_INT,
-  COGL_BOXED_FLOAT,
-  COGL_BOXED_MATRIX
-} CoglBoxedType;
-
-typedef struct _CoglBoxedValue
-{
-  CoglBoxedType type;
-  int size, count;
-  gboolean transpose;
-
-  union {
-    float float_value[4];
-    int int_value[4];
-    float matrix[16];
-    float *float_array;
-    int *int_array;
-    void *array;
-  } v;
-} CoglBoxedValue;
-
 #ifdef COGL_GL_DEBUG
 
 const char *
@@ -98,10 +67,6 @@ cogl_gl_error_to_string (GLenum error_code);
 #define COGL_ENABLE_ALPHA_TEST        (1<<1)
 #define COGL_ENABLE_VERTEX_ARRAY      (1<<2)
 #define COGL_ENABLE_COLOR_ARRAY       (1<<3)
-#define COGL_ENABLE_BACKFACE_CULLING  (1<<4)
-
-int
-_cogl_get_format_bpp (CoglPixelFormat format);
 
 void
 _cogl_enable (unsigned long flags);
@@ -109,9 +74,6 @@ _cogl_enable (unsigned long flags);
 unsigned long
 _cogl_get_enable (void);
 
-void
-_cogl_flush_face_winding (void);
-
 void
 _cogl_transform_point (const CoglMatrix *matrix_mv,
                        const CoglMatrix *matrix_p,
@@ -138,9 +100,22 @@ typedef enum
 typedef enum
 {
   COGL_PRIVATE_FEATURE_TEXTURE_2D_FROM_EGL_IMAGE = 1L<<0,
-  COGL_PRIVATE_FEATURE_MESA_PACK_INVERT = 1L<<1
+  COGL_PRIVATE_FEATURE_MESA_PACK_INVERT = 1L<<1,
+  COGL_PRIVATE_FEATURE_STENCIL_BUFFER = 1L<<2,
+  COGL_PRIVATE_FEATURE_OFFSCREEN_BLIT = 1L<<3,
+  COGL_PRIVATE_FEATURE_FOUR_CLIP_PLANES = 1L<<4,
+  COGL_PRIVATE_FEATURE_PBOS = 1L<<5,
+  COGL_PRIVATE_FEATURE_VBOS = 1L<<6
 } CoglPrivateFeatureFlags;
 
+/* Sometimes when evaluating pipelines, either during comparisons or
+ * if calculating a hash value we need to tweak the evaluation
+ * semantics */
+typedef enum _CoglPipelineEvalFlags
+{
+  COGL_PIPELINE_EVAL_FLAG_NONE = 0
+} CoglPipelineEvalFlags;
+
 gboolean
 _cogl_check_extension (const char *name, const char *ext);
 
