@@ -37,7 +37,6 @@
 
 #ifdef COGL_PIPELINE_FRAGEND_ARBFP
 
-#include "cogl.h"
 #include "cogl-internal.h"
 #include "cogl-context-private.h"
 #include "cogl-handle.h"
@@ -291,26 +290,31 @@ _cogl_pipeline_fragend_arbfp_start (CoglPipeline *pipeline,
 }
 
 static const char *
-gl_target_to_arbfp_string (GLenum gl_target)
+texture_type_to_arbfp_string (CoglTextureType texture_type)
 {
-  if (gl_target == GL_TEXTURE_1D)
-    return "1D";
-  else if (gl_target == GL_TEXTURE_2D)
-    return "2D";
-#ifdef GL_ARB_texture_rectangle
-  else if (gl_target == GL_TEXTURE_RECTANGLE_ARB)
-    return "RECT";
+  switch (texture_type)
+    {
+#if 0 /* TODO */
+    case COGL_TEXTURE_TYPE_1D:
+      return "1D";
 #endif
-  else if (gl_target == GL_TEXTURE_3D)
-    return "3D";
-  else
-    return "2D";
+    case COGL_TEXTURE_TYPE_2D:
+      return "2D";
+    case COGL_TEXTURE_TYPE_3D:
+      return "3D";
+    case COGL_TEXTURE_TYPE_RECTANGLE:
+      return "RECT";
+    }
+
+  g_warn_if_reached ();
+
+  return "2D";
 }
 
 static void
 setup_texture_source (CoglPipelineShaderState *shader_state,
                       int unit_index,
-                      GLenum gl_target)
+                      CoglTextureType texture_type)
 {
   if (!shader_state->unit_state[unit_index].sampled)
     {
@@ -329,7 +333,7 @@ setup_texture_source (CoglPipelineShaderState *shader_state,
                                 unit_index,
                                 unit_index,
                                 unit_index,
-                                gl_target_to_arbfp_string (gl_target));
+                                texture_type_to_arbfp_string (texture_type));
       shader_state->unit_state[unit_index].sampled = TRUE;
     }
 }
@@ -349,7 +353,7 @@ typedef struct _CoglPipelineFragendARBfpArg
 
   /* for type = TEXTURE */
   int texture_unit;
-  GLenum texture_target;
+  CoglTextureType texture_type;
 
   /* for type = CONSTANT */
   int constant_id;
@@ -386,14 +390,12 @@ setup_arg (CoglPipeline *pipeline,
            CoglPipelineLayer *layer,
            CoglBlendStringChannelMask mask,
            int arg_index,
-           GLint src,
+           CoglPipelineCombineSource src,
            GLint op,
            CoglPipelineFragendARBfpArg *arg)
 {
   CoglPipelineShaderState *shader_state = get_shader_state (pipeline);
   static const char *tmp_name[3] = { "tmp0", "tmp1", "tmp2" };
-  GLenum gl_target;
-  CoglHandle texture;
 
   switch (src)
     {
@@ -401,12 +403,9 @@ setup_arg (CoglPipeline *pipeline,
       arg->type = COGL_PIPELINE_FRAGEND_ARBFP_ARG_TYPE_TEXTURE;
       arg->name = "texel%d";
       arg->texture_unit = _cogl_pipeline_layer_get_unit_index (layer);
-      texture = _cogl_pipeline_layer_get_texture (layer);
-      if (texture)
-        cogl_texture_get_gl_texture (texture, NULL, &gl_target);
-      else
-        gl_target = GL_TEXTURE_2D;
-      setup_texture_source (shader_state, arg->texture_unit, gl_target);
+      setup_texture_source (shader_state,
+                            arg->texture_unit,
+                            _cogl_pipeline_layer_get_texture_type (layer));
       break;
     case COGL_PIPELINE_COMBINE_SOURCE_CONSTANT:
       {
@@ -433,16 +432,40 @@ setup_arg (CoglPipeline *pipeline,
       else
         arg->name = "output";
       break;
-    default: /* GL_TEXTURE0..N */
-      arg->type = COGL_PIPELINE_FRAGEND_ARBFP_ARG_TYPE_TEXTURE;
-      arg->name = "texture[%d]";
-      arg->texture_unit = src - GL_TEXTURE0;
-      texture = _cogl_pipeline_layer_get_texture (layer);
-      if (texture)
-        cogl_texture_get_gl_texture (texture, NULL, &gl_target);
-      else
-        gl_target = GL_TEXTURE_2D;
-      setup_texture_source (shader_state, arg->texture_unit, gl_target);
+    default: /* Sample the texture attached to a specific layer */
+      {
+        int layer_num = src - COGL_PIPELINE_COMBINE_SOURCE_TEXTURE0;
+        CoglPipelineGetLayerFlags flags = COGL_PIPELINE_GET_LAYER_NO_CREATE;
+        CoglPipelineLayer *other_layer =
+          _cogl_pipeline_get_layer_with_flags (pipeline, layer_num, flags);
+
+        if (other_layer == NULL)
+          {
+            static gboolean warning_seen = FALSE;
+            if (!warning_seen)
+              {
+                g_warning ("The application is trying to use a texture "
+                           "combine with a layer number that does not exist");
+                warning_seen = TRUE;
+              }
+            arg->type = COGL_PIPELINE_FRAGEND_ARBFP_ARG_TYPE_SIMPLE;
+            arg->name = "output";
+          }
+        else
+          {
+            CoglTextureType texture_type;
+
+            arg->type = COGL_PIPELINE_FRAGEND_ARBFP_ARG_TYPE_TEXTURE;
+            arg->name = "texture[%d]";
+            arg->texture_unit =
+              _cogl_pipeline_layer_get_unit_index (other_layer);
+            texture_type = _cogl_pipeline_layer_get_texture_type (other_layer);
+            setup_texture_source (shader_state,
+                                  arg->texture_unit,
+                                  texture_type);
+          }
+      }
+      break;
     }
 
   arg->swizzle = "";
