@@ -29,14 +29,13 @@
 #include "config.h"
 #endif
 
-#include "cogl.h"
-
 #include "cogl-debug.h"
 #include "cogl-pipeline-opengl-private.h"
 #include "cogl-pipeline-private.h"
 #include "cogl-context-private.h"
 #include "cogl-texture-private.h"
 #include "cogl-framebuffer-private.h"
+#include "cogl-offscreen.h"
 
 /* This is needed to set the color attribute on GLES2 */
 #ifdef HAVE_COGL_GLES2
@@ -779,16 +778,23 @@ flush_layers_common_gl_state_cb (CoglPipelineLayer *layer, void *user_data)
 
   if (layers_difference & COGL_PIPELINE_LAYER_STATE_TEXTURE_DATA)
     {
-      unsigned long state = COGL_PIPELINE_LAYER_STATE_TEXTURE_DATA;
-      CoglPipelineLayer *authority =
-        _cogl_pipeline_layer_get_authority (layer, state);
-      CoglTexture *texture;
+      CoglTexture *texture = _cogl_pipeline_layer_get_texture_real (layer);
       GLuint gl_texture;
       GLenum gl_target;
 
-      texture = (authority->texture == NULL ?
-                 ctx->default_gl_texture_2d_tex :
-                 authority->texture);
+      if (texture == NULL)
+        switch (_cogl_pipeline_layer_get_texture_type (layer))
+          {
+          case COGL_TEXTURE_TYPE_2D:
+            texture = COGL_TEXTURE (ctx->default_gl_texture_2d_tex);
+            break;
+          case COGL_TEXTURE_TYPE_3D:
+            texture = COGL_TEXTURE (ctx->default_gl_texture_3d_tex);
+            break;
+          case COGL_TEXTURE_TYPE_RECTANGLE:
+            texture = COGL_TEXTURE (ctx->default_gl_texture_rect_tex);
+            break;
+          }
 
       cogl_texture_get_gl_texture (texture,
                                    &gl_texture,
