@@ -31,7 +31,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "cogl.h"
 #include "cogl-util.h"
 #include "cogl-internal.h"
 #include "cogl-private.h"
@@ -489,8 +488,8 @@ cogl_renderer_get_n_fragment_texture_units (CoglRenderer *renderer)
 }
 
 void
-cogl_renderer_add_contraint (CoglRenderer *renderer,
-                             CoglRendererConstraint constraint)
+cogl_renderer_add_constraint (CoglRenderer *renderer,
+                              CoglRendererConstraint constraint)
 {
   g_return_if_fail (!renderer->connected);
   renderer->constraints = g_list_prepend (renderer->constraints,
