@@ -30,9 +30,8 @@
 #include "config.h"
 #endif
 
-#include "cogl.h"
 #include "cogl-debug.h"
-#include "cogl-internal.h"
+#include "cogl-private.h"
 #include "cogl-util.h"
 #include "cogl-bitmap.h"
 #include "cogl-bitmap-private.h"
@@ -146,7 +145,7 @@ _cogl_texture_2d_sliced_allocate_waste_buffer (CoglTexture2DSliced *tex_2ds,
                                 tex_2ds->slice_y_spans->len - 1);
   if (last_x_span->waste > 0 || last_y_span->waste > 0)
     {
-      int bpp = _cogl_get_format_bpp (format);
+      int bpp = _cogl_pixel_format_get_bytes_per_pixel (format);
       CoglSpan  *first_x_span
         = &g_array_index (tex_2ds->slice_x_spans, CoglSpan, 0);
       CoglSpan  *first_y_span
@@ -190,7 +189,7 @@ _cogl_texture_2d_sliced_set_waste (CoglTexture2DSliced *tex_2ds,
     {
       int bmp_rowstride = _cogl_bitmap_get_rowstride (source_bmp);
       CoglPixelFormat source_format = _cogl_bitmap_get_format (source_bmp);
-      int bpp = _cogl_get_format_bpp (source_format);
+      int bpp = _cogl_pixel_format_get_bytes_per_pixel (source_format);
       guint8 *bmp_data;
       const guint8 *src;
       guint8 *dst;
@@ -1293,6 +1292,12 @@ _cogl_texture_2d_sliced_get_height (CoglTexture *tex)
   return COGL_TEXTURE_2D_SLICED (tex)->height;
 }
 
+static CoglTextureType
+_cogl_texture_2d_sliced_get_type (CoglTexture *tex)
+{
+  return COGL_TEXTURE_TYPE_2D;
+}
+
 static const CoglTextureVtable
 cogl_texture_2d_sliced_vtable =
   {
@@ -1313,5 +1318,6 @@ cogl_texture_2d_sliced_vtable =
     _cogl_texture_2d_sliced_get_gl_format,
     _cogl_texture_2d_sliced_get_width,
     _cogl_texture_2d_sliced_get_height,
+    _cogl_texture_2d_sliced_get_type,
     _cogl_texture_2d_sliced_is_foreign
   };
