@@ -28,8 +28,7 @@
 #include "config.h"
 #endif
 
-#include "cogl.h"
-#include "cogl-internal.h"
+#include "cogl-private.h"
 #include "cogl-util.h"
 #include "cogl-texture-private.h"
 #include "cogl-texture-2d-private.h"
@@ -268,10 +267,11 @@ _cogl_texture_2d_new_from_bitmap (CoglBitmap      *bmp,
       (data = _cogl_bitmap_map (dst_bmp,
                                 COGL_BUFFER_ACCESS_READ, 0)))
     {
+      CoglPixelFormat format = _cogl_bitmap_get_format (dst_bmp);
       tex_2d->first_pixel.gl_format = gl_format;
       tex_2d->first_pixel.gl_type = gl_type;
       memcpy (tex_2d->first_pixel.data, data,
-              _cogl_get_format_bpp (_cogl_bitmap_get_format (dst_bmp)));
+              _cogl_pixel_format_get_bytes_per_pixel (format));
 
       _cogl_bitmap_unmap (dst_bmp);
     }
@@ -310,7 +310,7 @@ cogl_texture_2d_new_from_data (CoglContext *ctx,
 
   /* Rowstride from width if not given */
   if (rowstride == 0)
-    rowstride = width * _cogl_get_format_bpp (format);
+    rowstride = width * _cogl_pixel_format_get_bytes_per_pixel (format);
 
   /* Wrap the data into a bitmap */
   bmp = _cogl_bitmap_new_from_data ((guint8 *)data,
@@ -540,6 +540,7 @@ cogl_wayland_texture_2d_new_from_buffer (CoglContext *ctx,
   else
     {
       EGLImageKHR image;
+      CoglTexture2D *tex;
 
       _COGL_RETURN_VAL_IF_FAIL (_cogl_context_get_winsys (ctx)->constraints &
                                 COGL_RENDERER_CONSTRAINT_USES_EGL,
@@ -548,13 +549,14 @@ cogl_wayland_texture_2d_new_from_buffer (CoglContext *ctx,
                                       EGL_WAYLAND_BUFFER_WL,
                                       buffer,
                                       NULL);
-#warning "XXX: without a way to query the format of a wayland buffer we have to guess!"
-      return _cogl_egl_texture_2d_new_from_image (ctx,
-                                                  buffer->width,
-                                                  buffer->height,
-                                                  COGL_PIXEL_FORMAT_ARGB_8888_PRE,
-                                                  image,
-                                                  error);
+      tex = _cogl_egl_texture_2d_new_from_image (ctx,
+                                                 buffer->width,
+                                                 buffer->height,
+                                                 COGL_PIXEL_FORMAT_ARGB_8888_PRE,
+                                                 image,
+                                                 error);
+      _cogl_egl_destroy_image (ctx, image);
+      return tex;
     }
 }
 #endif /* COGL_HAS_WAYLAND_EGL_SERVER_SUPPORT */
@@ -768,7 +770,7 @@ _cogl_texture_2d_set_region (CoglTexture    *tex,
       (data = _cogl_bitmap_map (bmp, COGL_BUFFER_ACCESS_READ, 0)))
     {
       CoglPixelFormat bpp =
-        _cogl_get_format_bpp (_cogl_bitmap_get_format (bmp));
+        _cogl_pixel_format_get_bytes_per_pixel (_cogl_bitmap_get_format (bmp));
       tex_2d->first_pixel.gl_format = gl_format;
       tex_2d->first_pixel.gl_type = gl_type;
       memcpy (tex_2d->first_pixel.data,
@@ -809,7 +811,7 @@ _cogl_texture_2d_get_data (CoglTexture     *tex,
 
   _COGL_GET_CONTEXT (ctx, FALSE);
 
-  bpp = _cogl_get_format_bpp (format);
+  bpp = _cogl_pixel_format_get_bytes_per_pixel (format);
 
   ctx->texture_driver->pixel_format_to_gl (format,
                                            NULL, /* internal format */
@@ -857,6 +859,12 @@ _cogl_texture_2d_is_foreign (CoglTexture *tex)
   return COGL_TEXTURE_2D (tex)->is_foreign;
 }
 
+static CoglTextureType
+_cogl_texture_2d_get_type (CoglTexture *tex)
+{
+  return COGL_TEXTURE_TYPE_2D;
+}
+
 static const CoglTextureVtable
 cogl_texture_2d_vtable =
   {
@@ -877,5 +885,6 @@ cogl_texture_2d_vtable =
     _cogl_texture_2d_get_gl_format,
     _cogl_texture_2d_get_width,
     _cogl_texture_2d_get_height,
+    _cogl_texture_2d_get_type,
     _cogl_texture_2d_is_foreign
   };
