@@ -31,7 +31,6 @@
 #include "config.h"
 #endif
 
-#include "cogl.h"
 #include "cogl-internal.h"
 #include "cogl-util.h"
 #include "cogl-bitmap.h"
@@ -52,6 +51,8 @@
 #include "cogl-object-private.h"
 #include "cogl-primitives.h"
 #include "cogl-framebuffer-private.h"
+#include "cogl1-context.h"
+#include "cogl-sub-texture.h"
 
 #include <string.h>
 #include <stdlib.h>
@@ -70,12 +71,11 @@ cogl_texture_error_quark (void)
  */
 
 void
-_cogl_texture_register_texture_type (GQuark type)
+_cogl_texture_register_texture_type (const CoglObjectClass *klass)
 {
   _COGL_GET_CONTEXT (ctxt, NO_RETVAL);
 
-  ctxt->texture_types = g_slist_prepend (ctxt->texture_types,
-                                         GINT_TO_POINTER (type));
+  ctxt->texture_types = g_slist_prepend (ctxt->texture_types, (void *) klass);
 }
 
 gboolean
@@ -90,7 +90,7 @@ cogl_is_texture (void *object)
     return FALSE;
 
   for (l = ctxt->texture_types; l; l = l->next)
-    if (GPOINTER_TO_INT (l->data) == obj->klass->type)
+    if (l->data == obj->klass)
       return TRUE;
 
   return FALSE;
@@ -343,7 +343,7 @@ cogl_texture_new_from_data (unsigned int      width,
 
   /* Rowstride from width if not given */
   if (rowstride == 0)
-    rowstride = width * _cogl_get_format_bpp (format);
+    rowstride = width * _cogl_pixel_format_get_bytes_per_pixel (format);
 
   /* Wrap the data into a bitmap */
   bmp = _cogl_bitmap_new_from_data ((guint8 *) data,
@@ -530,7 +530,7 @@ cogl_texture_new_from_buffer_EXP (CoglPixelBuffer    *buffer,
   if (rowstride == 0)
     rowstride = pixel_buffer->stride;
   if (rowstride == 0)
-    rowstride = width * _cogl_get_format_bpp (format);
+    rowstride = width * _cogl_pixel_format_get_bytes_per_pixel (format);
 
   /* use the CoglBuffer height and width as last resort */
   if (width == 0)
@@ -580,12 +580,13 @@ cogl_texture_get_format (CoglTexture *texture)
 unsigned int
 cogl_texture_get_rowstride (CoglTexture *texture)
 {
+  CoglPixelFormat format = cogl_texture_get_format (texture);
   /* FIXME: This function should go away. It previously just returned
      the rowstride that was used to upload the data as far as I can
      tell. This is not helpful */
 
   /* Just guess at a suitable rowstride */
-  return (_cogl_get_format_bpp (cogl_texture_get_format (texture))
+  return (_cogl_pixel_format_get_bytes_per_pixel (format)
           * cogl_texture_get_width (texture));
 }
 
@@ -644,6 +645,12 @@ cogl_texture_get_gl_texture (CoglTexture *texture,
                                           out_gl_handle, out_gl_target);
 }
 
+CoglTextureType
+_cogl_texture_get_type (CoglTexture *texture)
+{
+  return texture->vtable->get_type (texture);
+}
+
 void
 _cogl_texture_set_filters (CoglTexture *texture,
                            GLenum min_filter,
@@ -727,7 +734,7 @@ cogl_texture_set_region (CoglTexture     *texture,
 
   /* Rowstride from width if none specified */
   if (rowstride == 0)
-    rowstride = _cogl_get_format_bpp (format) * width;
+    rowstride = _cogl_pixel_format_get_bytes_per_pixel (format) * width;
 
   /* Init source bitmap */
   source_bmp = _cogl_bitmap_new_from_data ((guint8 *) data,
@@ -776,7 +783,7 @@ do_texture_draw_and_read (CoglTexture *texture,
   CoglBitmap  *rect_bmp;
   unsigned int  tex_width, tex_height;
 
-  bpp = _cogl_get_format_bpp (COGL_PIXEL_FORMAT_RGBA_8888);
+  bpp = _cogl_pixel_format_get_bytes_per_pixel (COGL_PIXEL_FORMAT_RGBA_8888);
 
   tex_width = cogl_texture_get_width (texture);
   tex_height = cogl_texture_get_height (texture);
@@ -880,7 +887,7 @@ _cogl_texture_draw_and_read (CoglTexture *texture,
 
   _COGL_GET_CONTEXT (ctx, FALSE);
 
-  bpp = _cogl_get_format_bpp (COGL_PIXEL_FORMAT_RGBA_8888);
+  bpp = _cogl_pixel_format_get_bytes_per_pixel (COGL_PIXEL_FORMAT_RGBA_8888);
 
   framebuffer = cogl_get_draw_framebuffer ();
   /* Viewport needs to have some size and be inside the window for this */
@@ -907,7 +914,7 @@ _cogl_texture_draw_and_read (CoglTexture *texture,
 
   if (ctx->texture_download_pipeline == NULL)
     {
-      ctx->texture_download_pipeline = cogl_pipeline_new ();
+      ctx->texture_download_pipeline = cogl_pipeline_new (ctx);
       cogl_pipeline_set_blend (ctx->texture_download_pipeline,
                                "RGBA = ADD (SRC_COLOR, 0)",
                                NULL);
@@ -1012,6 +1019,7 @@ get_texture_bits_via_offscreen (CoglTexture    *texture,
                                 unsigned int    dst_rowstride,
                                 CoglPixelFormat dst_format)
 {
+  CoglOffscreen *offscreen;
   CoglFramebuffer *framebuffer;
 
   _COGL_GET_CONTEXT (ctx, FALSE);
@@ -1019,14 +1027,22 @@ get_texture_bits_via_offscreen (CoglTexture    *texture,
   if (!cogl_has_feature (ctx, COGL_FEATURE_ID_OFFSCREEN))
     return FALSE;
 
-  framebuffer = _cogl_offscreen_new_to_texture_full
+  offscreen = _cogl_offscreen_new_to_texture_full
                                       (texture,
                                        COGL_OFFSCREEN_DISABLE_DEPTH_AND_STENCIL,
                                        0);
 
-  if (framebuffer == NULL)
+  if (offscreen == NULL)
     return FALSE;
 
+  framebuffer = COGL_FRAMEBUFFER (offscreen);
+
+  if (!cogl_framebuffer_allocate (framebuffer, NULL))
+    {
+      cogl_object_unref (framebuffer);
+      return FALSE;
+    }
+
   cogl_push_framebuffer (framebuffer);
 
   _cogl_read_pixels_with_rowstride (x, y, width, height,
@@ -1059,7 +1075,7 @@ get_texture_bits_via_copy (CoglTexture    *texture,
   full_tex_width = cogl_texture_get_width (texture);
   full_tex_height = cogl_texture_get_height (texture);
 
-  bpp = _cogl_get_format_bpp (dst_format);
+  bpp = _cogl_pixel_format_get_bytes_per_pixel (dst_format);
 
   full_rowstride = bpp * full_tex_width;
   full_bits = g_malloc (full_rowstride * full_tex_height);
@@ -1105,7 +1121,7 @@ texture_get_cb (CoglTexture *texture,
 {
   CoglTextureGetData *tg_data = user_data;
   CoglPixelFormat format = _cogl_bitmap_get_format (tg_data->target_bmp);
-  int bpp = _cogl_get_format_bpp (format);
+  int bpp = _cogl_pixel_format_get_bytes_per_pixel (format);
   unsigned int rowstride = _cogl_bitmap_get_rowstride (tg_data->target_bmp);
   int subtexture_width = cogl_texture_get_width (texture);
   int subtexture_height = cogl_texture_get_height (texture);
@@ -1195,7 +1211,7 @@ cogl_texture_get_data (CoglTexture     *texture,
   tex_height = cogl_texture_get_height (texture);
 
   /* Rowstride from texture width if none specified */
-  bpp = _cogl_get_format_bpp (format);
+  bpp = _cogl_pixel_format_get_bytes_per_pixel (format);
   if (rowstride == 0)
     rowstride = tex_width * bpp;
 
@@ -1208,7 +1224,7 @@ cogl_texture_get_data (CoglTexture     *texture,
     ctx->texture_driver->find_best_gl_get_data_format (format,
                                                        &closest_gl_format,
                                                        &closest_gl_type);
-  closest_bpp = _cogl_get_format_bpp (closest_format);
+  closest_bpp = _cogl_pixel_format_get_bytes_per_pixel (closest_format);
 
   /* Is the requested format supported? */
   if (closest_format == format)
@@ -1244,6 +1260,11 @@ cogl_texture_get_data (CoglTexture     *texture,
     }
   tg_data.success = TRUE;
 
+  /* If there are any dependent framebuffers on the texture then we
+     need to flush their journals so the texture contents will be
+     up-to-date */
+  _cogl_texture_flush_journal_rendering (texture);
+
   /* Iterating through the subtextures allows piecing together
    * the data for a sliced texture, and allows us to do the
    * read-from-framebuffer logic here in a simple fashion rather than
