@@ -28,6 +28,11 @@
 #ifndef __COGL_TEXTURE_H__
 #define __COGL_TEXTURE_H__
 
+/* We forward declare the CoglTexture type here to avoid some circular
+ * dependency issues with the following headers.
+ */
+typedef struct _CoglTexture CoglTexture;
+
 #include <cogl/cogl-types.h>
 #include <cogl/cogl-defines.h>
 #if defined (COGL_ENABLE_EXPERIMENTAL_API)
@@ -46,7 +51,6 @@ G_BEGIN_DECLS
  * loading and manipulating textures.
  */
 
-typedef struct _CoglTexture CoglTexture;
 #define COGL_TEXTURE(X) ((CoglTexture *)X)
 
 #define COGL_TEXTURE_MAX_WASTE  127
@@ -78,6 +82,24 @@ typedef enum {
   COGL_TEXTURE_ERROR_TYPE
 } CoglTextureError;
 
+/**
+ * CoglTextureType:
+ * @COGL_TEXTURE_TYPE_2D: A #CoglTexture2D
+ * @COGL_TEXTURE_TYPE_3D: A #CoglTexture3D
+ * @COGL_TEXTURE_TYPE_RECTANGLE: A #CoglTextureRectangle
+ *
+ * Constants representing the underlying hardware texture type of a
+ * #CoglTexture.
+ *
+ * Stability: unstable
+ * Since: 1.10
+ */
+typedef enum {
+  COGL_TEXTURE_TYPE_2D,
+  COGL_TEXTURE_TYPE_3D,
+  COGL_TEXTURE_TYPE_RECTANGLE
+} CoglTextureType;
+
 GQuark cogl_texture_error_quark (void);
 
 /**
