@@ -30,8 +30,7 @@
 #include "config.h"
 #endif
 
-#include "cogl.h"
-#include "cogl-internal.h"
+#include "cogl-private.h"
 #include "cogl-util.h"
 #include "cogl-bitmap.h"
 #include "cogl-bitmap-private.h"
@@ -158,7 +157,8 @@ _cogl_texture_driver_upload_subregion_to_gl (GLenum       gl_target,
 				             GLuint       source_gl_type)
 {
   guint8 *data;
-  int bpp = _cogl_get_format_bpp (_cogl_bitmap_get_format (source_bmp));
+  CoglPixelFormat source_format = _cogl_bitmap_get_format (source_bmp);
+  int bpp = _cogl_pixel_format_get_bytes_per_pixel (source_format);
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
@@ -193,7 +193,8 @@ _cogl_texture_driver_upload_to_gl (GLenum       gl_target,
                                    GLuint       source_gl_type)
 {
   guint8 *data;
-  int bpp = _cogl_get_format_bpp (_cogl_bitmap_get_format (source_bmp));
+  CoglPixelFormat source_format = _cogl_bitmap_get_format (source_bmp);
+  int bpp = _cogl_pixel_format_get_bytes_per_pixel (source_format);
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
@@ -229,7 +230,8 @@ _cogl_texture_driver_upload_to_gl_3d (GLenum       gl_target,
                                       GLuint       source_gl_type)
 {
   guint8 *data;
-  int bpp = _cogl_get_format_bpp (_cogl_bitmap_get_format (source_bmp));
+  CoglPixelFormat source_format = _cogl_bitmap_get_format (source_bmp);
+  int bpp = _cogl_pixel_format_get_bytes_per_pixel (source_format);
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
@@ -397,16 +399,14 @@ _cogl_texture_driver_pixel_format_to_gl (CoglPixelFormat  format,
                                          GLenum          *out_gltype)
 {
   CoglPixelFormat required_format;
-  GLenum          glintformat = 0;
-  GLenum          glformat = 0;
-  GLenum          gltype = 0;
-
-  /* FIXME: check YUV support */
+  GLenum glintformat;
+  GLenum glformat;
+  GLenum gltype;
 
   required_format = format;
 
   /* Find GL equivalents */
-  switch (format & COGL_UNPREMULT_MASK)
+  switch (format)
     {
     case COGL_PIXEL_FORMAT_A_8:
       glintformat = GL_ALPHA;
@@ -430,11 +430,13 @@ _cogl_texture_driver_pixel_format_to_gl (CoglPixelFormat  format,
       gltype = GL_UNSIGNED_BYTE;
       break;
     case COGL_PIXEL_FORMAT_RGBA_8888:
+    case COGL_PIXEL_FORMAT_RGBA_8888_PRE:
       glintformat = GL_RGBA;
       glformat = GL_RGBA;
       gltype = GL_UNSIGNED_BYTE;
       break;
     case COGL_PIXEL_FORMAT_BGRA_8888:
+    case COGL_PIXEL_FORMAT_BGRA_8888_PRE:
       glintformat = GL_RGBA;
       glformat = GL_BGRA;
       gltype = GL_UNSIGNED_BYTE;
@@ -444,6 +446,7 @@ _cogl_texture_driver_pixel_format_to_gl (CoglPixelFormat  format,
        * have no GL equivalent unless defined using
        * system word byte ordering */
     case COGL_PIXEL_FORMAT_ARGB_8888:
+    case COGL_PIXEL_FORMAT_ARGB_8888_PRE:
       glintformat = GL_RGBA;
       glformat = GL_BGRA;
 #if G_BYTE_ORDER == G_LITTLE_ENDIAN
@@ -454,6 +457,7 @@ _cogl_texture_driver_pixel_format_to_gl (CoglPixelFormat  format,
       break;
 
     case COGL_PIXEL_FORMAT_ABGR_8888:
+    case COGL_PIXEL_FORMAT_ABGR_8888_PRE:
       glintformat = GL_RGBA;
       glformat = GL_RGBA;
 #if G_BYTE_ORDER == G_LITTLE_ENDIAN
@@ -463,6 +467,34 @@ _cogl_texture_driver_pixel_format_to_gl (CoglPixelFormat  format,
 #endif
       break;
 
+    case COGL_PIXEL_FORMAT_RGBA_1010102:
+    case COGL_PIXEL_FORMAT_RGBA_1010102_PRE:
+      glintformat = GL_RGBA;
+      glformat = GL_RGBA;
+      gltype = GL_UNSIGNED_INT_10_10_10_2;
+      break;
+
+    case COGL_PIXEL_FORMAT_BGRA_1010102:
+    case COGL_PIXEL_FORMAT_BGRA_1010102_PRE:
+      glintformat = GL_RGBA;
+      glformat = GL_BGRA;
+      gltype = GL_UNSIGNED_INT_10_10_10_2;
+      break;
+
+    case COGL_PIXEL_FORMAT_ABGR_2101010:
+    case COGL_PIXEL_FORMAT_ABGR_2101010_PRE:
+      glintformat = GL_RGBA;
+      glformat = GL_RGBA;
+      gltype = GL_UNSIGNED_INT_2_10_10_10_REV;
+      break;
+
+    case COGL_PIXEL_FORMAT_ARGB_2101010:
+    case COGL_PIXEL_FORMAT_ARGB_2101010_PRE:
+      glintformat = GL_RGBA;
+      glformat = GL_BGRA;
+      gltype = GL_UNSIGNED_INT_2_10_10_10_REV;
+      break;
+
       /* The following three types of channel ordering
        * are always defined using system word byte
        * ordering (even according to GLES spec) */
@@ -472,18 +504,21 @@ _cogl_texture_driver_pixel_format_to_gl (CoglPixelFormat  format,
       gltype = GL_UNSIGNED_SHORT_5_6_5;
       break;
     case COGL_PIXEL_FORMAT_RGBA_4444:
+    case COGL_PIXEL_FORMAT_RGBA_4444_PRE:
       glintformat = GL_RGBA;
       glformat = GL_RGBA;
       gltype = GL_UNSIGNED_SHORT_4_4_4_4;
       break;
     case COGL_PIXEL_FORMAT_RGBA_5551:
+    case COGL_PIXEL_FORMAT_RGBA_5551_PRE:
       glintformat = GL_RGBA;
       glformat = GL_RGBA;
       gltype = GL_UNSIGNED_SHORT_5_5_5_1;
       break;
 
-      /* FIXME: check extensions for YUV support */
-    default:
+    case COGL_PIXEL_FORMAT_ANY:
+    case COGL_PIXEL_FORMAT_YUV:
+      g_assert_not_reached ();
       break;
     }
 
