@@ -30,8 +30,7 @@
 #include "config.h"
 #endif
 
-#include "cogl.h"
-#include "cogl-internal.h"
+#include "cogl-private.h"
 #include "cogl-util.h"
 #include "cogl-bitmap.h"
 #include "cogl-bitmap-private.h"
@@ -46,9 +45,6 @@
 #include <stdlib.h>
 #include <math.h>
 
-#define glTexImage3D ctx->drv.pf_glTexImage3D
-#define glTexSubImage3D ctx->drv.pf_glTexSubImage3D
-
 #ifndef GL_TEXTURE_3D
 #define GL_TEXTURE_3D 0x806F
 #endif
@@ -56,14 +52,16 @@
 #define GL_MAX_3D_TEXTURE_SIZE_OES 0x8073
 #endif
 
-void
+static void
 _cogl_texture_driver_gen (GLenum   gl_target,
                           GLsizei  n,
                           GLuint  *textures)
 {
   unsigned int i;
 
-  GE (glGenTextures (n, textures));
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  GE (ctx, glGenTextures (n, textures));
 
   for (i = 0; i < n; i++)
     {
@@ -74,7 +72,9 @@ _cogl_texture_driver_gen (GLenum   gl_target,
         case GL_TEXTURE_2D:
         case GL_TEXTURE_3D:
           /* GL_TEXTURE_MAG_FILTER defaults to GL_LINEAR, no need to set it */
-          GE( glTexParameteri (gl_target, GL_TEXTURE_MIN_FILTER, GL_LINEAR) );
+          GE( ctx, glTexParameteri (gl_target,
+                                    GL_TEXTURE_MIN_FILTER,
+                                    GL_LINEAR) );
           break;
 
         default:
@@ -83,14 +83,14 @@ _cogl_texture_driver_gen (GLenum   gl_target,
     }
 }
 
-void
+static void
 _cogl_texture_driver_prep_gl_for_pixels_upload (int pixels_rowstride,
                                                 int pixels_bpp)
 {
   _cogl_texture_prep_gl_alignment_for_pixels_upload (pixels_rowstride);
 }
 
-void
+static void
 _cogl_texture_driver_prep_gl_for_pixels_download (int pixels_rowstride,
                                                   int pixels_bpp)
 {
@@ -101,7 +101,7 @@ static CoglBitmap *
 prepare_bitmap_alignment_for_upload (CoglBitmap *src_bmp)
 {
   CoglPixelFormat format = _cogl_bitmap_get_format (src_bmp);
-  int bpp = _cogl_get_format_bpp (format);
+  int bpp = _cogl_pixel_format_get_bytes_per_pixel (format);
   int src_rowstride = _cogl_bitmap_get_rowstride (src_bmp);
   int width = _cogl_bitmap_get_width (src_bmp);
   int alignment = 1;
@@ -123,7 +123,7 @@ prepare_bitmap_alignment_for_upload (CoglBitmap *src_bmp)
     return _cogl_bitmap_copy (src_bmp);
 }
 
-void
+static void
 _cogl_texture_driver_upload_subregion_to_gl (GLenum       gl_target,
                                              GLuint       gl_handle,
                                              gboolean     is_foreign,
@@ -139,10 +139,12 @@ _cogl_texture_driver_upload_subregion_to_gl (GLenum       gl_target,
 {
   guint8 *data;
   CoglPixelFormat source_format = _cogl_bitmap_get_format (source_bmp);
-  int bpp = _cogl_get_format_bpp (source_format);
+  int bpp = _cogl_pixel_format_get_bytes_per_pixel (source_format);
   CoglBitmap *slice_bmp;
   int rowstride;
 
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
   /* If we are copying a sub region of the source bitmap then we need
      to copy it because GLES does not support GL_UNPACK_ROW_LENGTH */
   if (src_x != 0 || src_y != 0 ||
@@ -177,19 +179,19 @@ _cogl_texture_driver_upload_subregion_to_gl (GLenum       gl_target,
 
   _cogl_bind_gl_texture_transient (gl_target, gl_handle, is_foreign);
 
-  GE( glTexSubImage2D (gl_target, 0,
-                       dst_x, dst_y,
-                       width, height,
-                       source_gl_format,
-                       source_gl_type,
-                       data) );
+  GE( ctx, glTexSubImage2D (gl_target, 0,
+                            dst_x, dst_y,
+                            width, height,
+                            source_gl_format,
+                            source_gl_type,
+                            data) );
 
   _cogl_bitmap_unbind (slice_bmp);
 
   cogl_object_unref (slice_bmp);
 }
 
-void
+static void
 _cogl_texture_driver_upload_to_gl (GLenum       gl_target,
                                    GLuint       gl_handle,
                                    gboolean     is_foreign,
@@ -198,13 +200,16 @@ _cogl_texture_driver_upload_to_gl (GLenum       gl_target,
                                    GLuint       source_gl_format,
                                    GLuint       source_gl_type)
 {
-  int bpp = _cogl_get_format_bpp (_cogl_bitmap_get_format (source_bmp));
+  CoglPixelFormat source_format = _cogl_bitmap_get_format (source_bmp);
+  int bpp = _cogl_pixel_format_get_bytes_per_pixel (source_format);
   int rowstride;
   int bmp_width = _cogl_bitmap_get_width (source_bmp);
   int bmp_height = _cogl_bitmap_get_height (source_bmp);
   CoglBitmap *bmp;
   guint8 *data;
 
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
   bmp = prepare_bitmap_alignment_for_upload (source_bmp);
   rowstride = _cogl_bitmap_get_rowstride (bmp);
 
@@ -215,20 +220,20 @@ _cogl_texture_driver_upload_to_gl (GLenum       gl_target,
 
   data = _cogl_bitmap_bind (bmp, COGL_BUFFER_ACCESS_READ, 0);
 
-  GE( glTexImage2D (gl_target, 0,
-                    internal_gl_format,
-                    bmp_width, bmp_height,
-                    0,
-                    source_gl_format,
-                    source_gl_type,
-                    data) );
+  GE( ctx, glTexImage2D (gl_target, 0,
+                         internal_gl_format,
+                         bmp_width, bmp_height,
+                         0,
+                         source_gl_format,
+                         source_gl_type,
+                         data) );
 
   _cogl_bitmap_unbind (bmp);
 
   cogl_object_unref (bmp);
 }
 
-void
+static void
 _cogl_texture_driver_upload_to_gl_3d (GLenum       gl_target,
                                       GLuint       gl_handle,
                                       gboolean     is_foreign,
@@ -239,7 +244,8 @@ _cogl_texture_driver_upload_to_gl_3d (GLenum       gl_target,
                                       GLuint       source_gl_format,
                                       GLuint       source_gl_type)
 {
-  int bpp = _cogl_get_format_bpp (_cogl_bitmap_get_format (source_bmp));
+  CoglPixelFormat source_format = _cogl_bitmap_get_format (source_bmp);
+  int bpp = _cogl_pixel_format_get_bytes_per_pixel (source_format);
   int rowstride = _cogl_bitmap_get_rowstride (source_bmp);
   int bmp_width = _cogl_bitmap_get_width (source_bmp);
   int bmp_height = _cogl_bitmap_get_height (source_bmp);
@@ -264,16 +270,16 @@ _cogl_texture_driver_upload_to_gl_3d (GLenum       gl_target,
       /* Initialize the texture with empty data and then upload each
          image with a sub-region update */
 
-      GE( glTexImage3D (gl_target,
-                        0, /* level */
-                        internal_gl_format,
-                        bmp_width,
-                        height,
-                        depth,
-                        0,
-                        source_gl_format,
-                        source_gl_type,
-                        NULL) );
+      GE( ctx, glTexImage3D (gl_target,
+                             0, /* level */
+                             internal_gl_format,
+                             bmp_width,
+                             height,
+                             depth,
+                             0,
+                             source_gl_format,
+                             source_gl_type,
+                             NULL) );
 
       bmp = _cogl_bitmap_new_from_data (g_malloc (bpp * bmp_width * height),
                                         _cogl_bitmap_get_format (source_bmp),
@@ -295,17 +301,17 @@ _cogl_texture_driver_upload_to_gl_3d (GLenum       gl_target,
           data = _cogl_bitmap_bind (bmp,
                                     COGL_BUFFER_ACCESS_READ, 0);
 
-          GE( glTexSubImage3D (gl_target,
-                               0, /* level */
-                               0, /* xoffset */
-                               0, /* yoffset */
-                               i, /* zoffset */
-                               bmp_width, /* width */
-                               height, /* height */
-                               1, /* depth */
-                               source_gl_format,
-                               source_gl_type,
-                               data) );
+          GE( ctx, glTexSubImage3D (gl_target,
+                                    0, /* level */
+                                    0, /* xoffset */
+                                    0, /* yoffset */
+                                    i, /* zoffset */
+                                    bmp_width, /* width */
+                                    height, /* height */
+                                    1, /* depth */
+                                    source_gl_format,
+                                    source_gl_type,
+                                    data) );
 
           _cogl_bitmap_unbind (bmp);
         }
@@ -318,16 +324,16 @@ _cogl_texture_driver_upload_to_gl_3d (GLenum       gl_target,
 
       _cogl_texture_driver_prep_gl_for_pixels_upload (rowstride, bpp);
 
-      GE( glTexImage3D (gl_target,
-                        0, /* level */
-                        internal_gl_format,
-                        bmp_width,
-                        height,
-                        depth,
-                        0,
-                        source_gl_format,
-                        source_gl_type,
-                        data) );
+      GE( ctx, glTexImage3D (gl_target,
+                             0, /* level */
+                             internal_gl_format,
+                             bmp_width,
+                             height,
+                             depth,
+                             0,
+                             source_gl_format,
+                             source_gl_type,
+                             data) );
 
       _cogl_bitmap_unbind (source_bmp);
     }
@@ -336,7 +342,7 @@ _cogl_texture_driver_upload_to_gl_3d (GLenum       gl_target,
 /* NB: GLES doesn't support glGetTexImage2D, so cogl-texture will instead
  * fallback to a generic render + readpixels approach to downloading
  * texture data. (See _cogl_texture_draw_and_read() ) */
-gboolean
+static gboolean
 _cogl_texture_driver_gl_get_tex_image (GLenum  gl_target,
                                        GLenum  dest_gl_format,
                                        GLenum  dest_gl_type,
@@ -345,7 +351,7 @@ _cogl_texture_driver_gl_get_tex_image (GLenum  gl_target,
   return FALSE;
 }
 
-gboolean
+static gboolean
 _cogl_texture_driver_size_supported_3d (GLenum gl_target,
                                         GLenum gl_format,
                                         GLenum gl_type,
@@ -355,15 +361,17 @@ _cogl_texture_driver_size_supported_3d (GLenum gl_target,
 {
   GLint max_size;
 
+  _COGL_GET_CONTEXT (ctx, FALSE);
+
   /* GLES doesn't support a proxy texture target so let's at least
      check whether the size is greater than
      GL_MAX_3D_TEXTURE_SIZE_OES */
-  GE( glGetIntegerv (GL_MAX_3D_TEXTURE_SIZE_OES, &max_size) );
+  GE( ctx, glGetIntegerv (GL_MAX_3D_TEXTURE_SIZE_OES, &max_size) );
 
   return width <= max_size && height <= max_size && depth <= max_size;
 }
 
-gboolean
+static gboolean
 _cogl_texture_driver_size_supported (GLenum gl_target,
                                      GLenum gl_format,
                                      GLenum gl_type,
@@ -372,14 +380,16 @@ _cogl_texture_driver_size_supported (GLenum gl_target,
 {
   GLint max_size;
 
+  _COGL_GET_CONTEXT (ctx, FALSE);
+
   /* GLES doesn't support a proxy texture target so let's at least
      check whether the size is greater than GL_MAX_TEXTURE_SIZE */
-  GE( glGetIntegerv (GL_MAX_TEXTURE_SIZE, &max_size) );
+  GE( ctx, glGetIntegerv (GL_MAX_TEXTURE_SIZE, &max_size) );
 
   return width <= max_size && height <= max_size;
 }
 
-void
+static void
 _cogl_texture_driver_try_setting_gl_border_color (
                                               GLuint   gl_target,
                                               const GLfloat *transparent_color)
@@ -387,30 +397,28 @@ _cogl_texture_driver_try_setting_gl_border_color (
   /* FAIL! */
 }
 
-gboolean
-_cogl_pixel_format_from_gl_internal (GLenum            gl_int_format,
-				     CoglPixelFormat  *out_format)
+static gboolean
+_cogl_texture_driver_pixel_format_from_gl_internal (GLenum gl_int_format,
+                                                    CoglPixelFormat *out_format)
 {
   return TRUE;
 }
 
-CoglPixelFormat
-_cogl_pixel_format_to_gl (CoglPixelFormat  format,
-			  GLenum          *out_glintformat,
-			  GLenum          *out_glformat,
-			  GLenum          *out_gltype)
+static CoglPixelFormat
+_cogl_texture_driver_pixel_format_to_gl (CoglPixelFormat  format,
+                                         GLenum          *out_glintformat,
+                                         GLenum          *out_glformat,
+                                         GLenum          *out_gltype)
 {
   CoglPixelFormat required_format;
-  GLenum          glintformat = 0;
-  GLenum          glformat = 0;
-  GLenum          gltype = 0;
-
-  /* FIXME: check YUV support */
+  GLenum glintformat;
+  GLenum glformat;
+  GLenum gltype;
 
   required_format = format;
 
   /* Find GL equivalents */
-  switch (format & COGL_UNPREMULT_MASK)
+  switch (format)
     {
     case COGL_PIXEL_FORMAT_A_8:
       glintformat = GL_ALPHA;
@@ -434,9 +442,21 @@ _cogl_pixel_format_to_gl (CoglPixelFormat  format,
 
       /* Just one 32-bit ordering supported */
     case COGL_PIXEL_FORMAT_RGBA_8888:
+    case COGL_PIXEL_FORMAT_RGBA_8888_PRE:
     case COGL_PIXEL_FORMAT_BGRA_8888:
+    case COGL_PIXEL_FORMAT_BGRA_8888_PRE:
     case COGL_PIXEL_FORMAT_ARGB_8888:
+    case COGL_PIXEL_FORMAT_ARGB_8888_PRE:
     case COGL_PIXEL_FORMAT_ABGR_8888:
+    case COGL_PIXEL_FORMAT_ABGR_8888_PRE:
+    case COGL_PIXEL_FORMAT_RGBA_1010102:
+    case COGL_PIXEL_FORMAT_RGBA_1010102_PRE:
+    case COGL_PIXEL_FORMAT_BGRA_1010102:
+    case COGL_PIXEL_FORMAT_BGRA_1010102_PRE:
+    case COGL_PIXEL_FORMAT_ABGR_2101010:
+    case COGL_PIXEL_FORMAT_ABGR_2101010_PRE:
+    case COGL_PIXEL_FORMAT_ARGB_2101010:
+    case COGL_PIXEL_FORMAT_ARGB_2101010_PRE:
       glintformat = GL_RGBA;
       glformat = GL_RGBA;
       gltype = GL_UNSIGNED_BYTE;
@@ -453,18 +473,21 @@ _cogl_pixel_format_to_gl (CoglPixelFormat  format,
       gltype = GL_UNSIGNED_SHORT_5_6_5;
       break;
     case COGL_PIXEL_FORMAT_RGBA_4444:
+    case COGL_PIXEL_FORMAT_RGBA_4444_PRE:
       glintformat = GL_RGBA;
       glformat = GL_RGBA;
       gltype = GL_UNSIGNED_SHORT_4_4_4_4;
       break;
     case COGL_PIXEL_FORMAT_RGBA_5551:
+    case COGL_PIXEL_FORMAT_RGBA_5551_PRE:
       glintformat = GL_RGBA;
       glformat = GL_RGBA;
       gltype = GL_UNSIGNED_SHORT_5_5_5_1;
       break;
 
-      /* FIXME: check extensions for YUV support */
-    default:
+    case COGL_PIXEL_FORMAT_ANY:
+    case COGL_PIXEL_FORMAT_YUV:
+      g_assert_not_reached ();
       break;
     }
 
@@ -478,7 +501,7 @@ _cogl_pixel_format_to_gl (CoglPixelFormat  format,
   return required_format;
 }
 
-gboolean
+static gboolean
 _cogl_texture_driver_allows_foreign_gl_target (GLenum gl_target)
 {
   /* Allow 2-dimensional textures only */
@@ -487,15 +510,16 @@ _cogl_texture_driver_allows_foreign_gl_target (GLenum gl_target)
   return TRUE;
 }
 
-void
+static void
 _cogl_texture_driver_gl_generate_mipmaps (GLenum gl_target)
 {
-#ifdef HAVE_COGL_GLES2
-  GE( glGenerateMipmap (gl_target) );
-#endif
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  if (ctx->driver == COGL_DRIVER_GLES2)
+    GE( ctx, glGenerateMipmap (gl_target) );
 }
 
-CoglPixelFormat
+static CoglPixelFormat
 _cogl_texture_driver_find_best_gl_get_data_format (
                                              CoglPixelFormat  format,
                                              GLenum          *closest_gl_format,
@@ -509,3 +533,22 @@ _cogl_texture_driver_find_best_gl_get_data_format (
   return COGL_PIXEL_FORMAT_RGBA_8888;
 }
 
+const CoglTextureDriver
+_cogl_texture_driver_gles =
+  {
+    _cogl_texture_driver_gen,
+    _cogl_texture_driver_prep_gl_for_pixels_upload,
+    _cogl_texture_driver_upload_subregion_to_gl,
+    _cogl_texture_driver_upload_to_gl,
+    _cogl_texture_driver_upload_to_gl_3d,
+    _cogl_texture_driver_prep_gl_for_pixels_download,
+    _cogl_texture_driver_gl_get_tex_image,
+    _cogl_texture_driver_size_supported,
+    _cogl_texture_driver_size_supported_3d,
+    _cogl_texture_driver_try_setting_gl_border_color,
+    _cogl_texture_driver_pixel_format_from_gl_internal,
+    _cogl_texture_driver_pixel_format_to_gl,
+    _cogl_texture_driver_allows_foreign_gl_target,
+    _cogl_texture_driver_gl_generate_mipmaps,
+    _cogl_texture_driver_find_best_gl_get_data_format
+  };
