@@ -28,8 +28,6 @@
 #include "config.h"
 #endif
 
-#include "cogl.h"
-
 #include "cogl-util.h"
 #include "cogl-winsys-private.h"
 #include "cogl-feature-private.h"
@@ -81,15 +79,9 @@ typedef struct _CoglOnscreenGLX
   GLXDrawable glxwin;
   guint32 last_swap_vsync_counter;
   GList *swap_callbacks;
+  gboolean pending_swap_notify;
 } CoglOnscreenGLX;
 
-typedef struct _CoglSwapBuffersNotifyEntry
-{
-  CoglSwapBuffersNotify callback;
-  void *user_data;
-  unsigned int id;
-} CoglSwapBuffersNotifyEntry;
-
 typedef struct _CoglTexturePixmapGLX
 {
   GLXPixmap glx_pixmap;
@@ -170,19 +162,20 @@ static void
 notify_swap_buffers (CoglContext *context, GLXDrawable drawable)
 {
   CoglOnscreen *onscreen = find_onscreen_for_xid (context, (guint32)drawable);
+  CoglDisplay *display = context->display;
+  CoglGLXDisplay *glx_display = display->winsys;
   CoglOnscreenGLX *glx_onscreen;
-  GList *l;
 
   if (!onscreen)
     return;
 
   glx_onscreen = onscreen->winsys;
 
-  for (l = glx_onscreen->swap_callbacks; l; l = l->next)
-    {
-      CoglSwapBuffersNotifyEntry *entry = l->data;
-      entry->callback (COGL_FRAMEBUFFER (onscreen), entry->user_data);
-    }
+  /* We only want to notify that the swap is complete when the
+     application calls cogl_context_dispatch so instead of immediately
+     notifying we'll set a flag to remember to notify later */
+  glx_display->pending_swap_notify = TRUE;
+  glx_onscreen->pending_swap_notify = TRUE;
 }
 
 static CoglFilterReturn
@@ -1360,45 +1353,6 @@ _cogl_winsys_onscreen_x11_get_window_xid (CoglOnscreen *onscreen)
   return xlib_onscreen->xwin;
 }
 
-static unsigned int
-_cogl_winsys_onscreen_add_swap_buffers_callback (CoglOnscreen *onscreen,
-                                                 CoglSwapBuffersNotify callback,
-                                                 void *user_data)
-{
-  CoglOnscreenGLX *glx_onscreen = onscreen->winsys;
-  CoglSwapBuffersNotifyEntry *entry = g_slice_new0 (CoglSwapBuffersNotifyEntry);
-  static int next_swap_buffers_callback_id = 0;
-
-  entry->callback = callback;
-  entry->user_data = user_data;
-  entry->id = next_swap_buffers_callback_id++;
-
-  glx_onscreen->swap_callbacks =
-    g_list_prepend (glx_onscreen->swap_callbacks, entry);
-
-  return entry->id;
-}
-
-static void
-_cogl_winsys_onscreen_remove_swap_buffers_callback (CoglOnscreen *onscreen,
-                                                    unsigned int id)
-{
-  CoglOnscreenGLX *glx_onscreen = onscreen->winsys;
-  GList *l;
-
-  for (l = glx_onscreen->swap_callbacks; l; l = l->next)
-    {
-      CoglSwapBuffersNotifyEntry *entry = l->data;
-      if (entry->id == id)
-        {
-          g_slice_free (CoglSwapBuffersNotifyEntry, entry);
-          glx_onscreen->swap_callbacks =
-            g_list_delete_link (glx_onscreen->swap_callbacks, l);
-          return;
-        }
-    }
-}
-
 static void
 _cogl_winsys_onscreen_update_swap_throttled (CoglOnscreen *onscreen)
 {
@@ -2013,10 +1967,37 @@ _cogl_winsys_poll_get_info (CoglContext *context,
                             int *n_poll_fds,
                             gint64 *timeout)
 {
+  CoglDisplay *display = context->display;
+  CoglGLXDisplay *glx_display = display->winsys;
+
   _cogl_xlib_renderer_poll_get_info (context->display->renderer,
                                      poll_fds,
                                      n_poll_fds,
                                      timeout);
+
+  /* If we've already got a pending swap notify then we'll dispatch
+     immediately */
+  if (glx_display->pending_swap_notify)
+    *timeout = 0;
+}
+
+static void
+flush_pending_swap_notify_cb (void *data,
+                              void *user_data)
+{
+  CoglFramebuffer *framebuffer = data;
+
+  if (framebuffer->type == COGL_FRAMEBUFFER_TYPE_ONSCREEN)
+    {
+      CoglOnscreen *onscreen = COGL_ONSCREEN (framebuffer);
+      CoglOnscreenGLX *glx_onscreen = onscreen->winsys;
+
+      if (glx_onscreen->pending_swap_notify)
+        {
+          _cogl_onscreen_notify_swap_buffers (onscreen);
+          glx_onscreen->pending_swap_notify = FALSE;
+        }
+    }
 }
 
 static void
@@ -2024,9 +2005,20 @@ _cogl_winsys_poll_dispatch (CoglContext *context,
                             const CoglPollFD *poll_fds,
                             int n_poll_fds)
 {
+  CoglDisplay *display = context->display;
+  CoglGLXDisplay *glx_display = display->winsys;
+
   _cogl_xlib_renderer_poll_dispatch (context->display->renderer,
                                      poll_fds,
                                      n_poll_fds);
+
+  if (glx_display->pending_swap_notify)
+    {
+      g_list_foreach (context->framebuffers,
+                      flush_pending_swap_notify_cb,
+                      NULL);
+      glx_display->pending_swap_notify = FALSE;
+    }
 }
 
 static CoglWinsysVtable _cogl_winsys_vtable =
@@ -2053,10 +2045,6 @@ static CoglWinsysVtable _cogl_winsys_vtable =
       _cogl_winsys_onscreen_update_swap_throttled,
     .onscreen_x11_get_window_xid =
       _cogl_winsys_onscreen_x11_get_window_xid,
-    .onscreen_add_swap_buffers_callback =
-      _cogl_winsys_onscreen_add_swap_buffers_callback,
-    .onscreen_remove_swap_buffers_callback =
-      _cogl_winsys_onscreen_remove_swap_buffers_callback,
     .onscreen_set_visibility = _cogl_winsys_onscreen_set_visibility,
 
     .poll_get_info = _cogl_winsys_poll_get_info,
