@@ -10,6 +10,7 @@ main (int argc, char **argv)
     CoglContext *ctx;
     CoglOnscreen *onscreen;
     CoglFramebuffer *fb;
+    CoglPipeline *pipeline;
     GError *error = NULL;
     CoglVertexP2C4 triangle_vertices[] = {
         {0, 0.7, 0xff, 0x00, 0x00, 0x80},
@@ -25,28 +26,22 @@ main (int argc, char **argv)
     }
 
     onscreen = cogl_onscreen_new (ctx, 640, 480);
-    /* Eventually there will be an implicit allocate on first use so this
-     * will become optional... */
-    fb = COGL_FRAMEBUFFER (onscreen);
-    if (!cogl_framebuffer_allocate (fb, &error)) {
-        fprintf (stderr, "Failed to allocate framebuffer: %s\n", error->message);
-        return 1;
-    }
-
     cogl_onscreen_show (onscreen);
+    fb = COGL_FRAMEBUFFER (onscreen);
 
-    cogl_push_framebuffer (fb);
-
-    triangle = cogl_primitive_new_p2c4 (COGL_VERTICES_MODE_TRIANGLES,
+    triangle = cogl_primitive_new_p2c4 (ctx, COGL_VERTICES_MODE_TRIANGLES,
                                         3, triangle_vertices);
+
+    pipeline = cogl_pipeline_new (ctx);
+
     for (;;) {
         CoglPollFD *poll_fds;
         int n_poll_fds;
         gint64 timeout;
 
-        cogl_clear (&black, COGL_BUFFER_BIT_COLOR);
-        cogl_primitive_draw (triangle);
-        cogl_framebuffer_swap_buffers (fb);
+        cogl_framebuffer_clear4f (fb, COGL_BUFFER_BIT_COLOR, 0, 0, 0, 1);
+        cogl_framebuffer_draw_primitive (fb, pipeline, triangle);
+        cogl_onscreen_swap_buffers (onscreen);
 
         cogl_poll_get_info (ctx, &poll_fds, &n_poll_fds, &timeout);
         g_poll ((GPollFD *) poll_fds, n_poll_fds, 0);
