@@ -142,13 +142,15 @@ main (int argc, char **argv)
   ADD_TEST ("/cogl", test_cogl_color_mask);
   ADD_TEST ("/cogl", test_cogl_backface_culling);
 
+  ADD_TEST ("/cogl/pipeline", test_cogl_sparse_pipeline);
+
   UNPORTED_TEST ("/cogl/texture", test_cogl_npot_texture);
   UNPORTED_TEST ("/cogl/texture", test_cogl_multitexture);
   UNPORTED_TEST ("/cogl/texture", test_cogl_texture_mipmaps);
   ADD_TEST ("/cogl/texture", test_cogl_sub_texture);
   UNPORTED_TEST ("/cogl/texture", test_cogl_pixel_array);
   UNPORTED_TEST ("/cogl/texture", test_cogl_texture_rectangle);
-  UNPORTED_TEST ("/cogl/texture", test_cogl_texture_3d);
+  ADD_TEST ("/cogl/texture", test_cogl_texture_3d);
   ADD_TEST ("/cogl/texture", test_cogl_wrap_modes);
   UNPORTED_TEST ("/cogl/texture", test_cogl_texture_pixmap_x11);
   UNPORTED_TEST ("/cogl/texture", test_cogl_texture_get_set_data);
@@ -158,7 +160,7 @@ main (int argc, char **argv)
   UNPORTED_TEST ("/cogl/vertex-buffer", test_cogl_vertex_buffer_interleved);
   UNPORTED_TEST ("/cogl/vertex-buffer", test_cogl_vertex_buffer_mutability);
 
-  UNPORTED_TEST ("/cogl/vertex-array", test_cogl_primitive);
+  ADD_TEST ("/cogl/vertex-array", test_cogl_primitive);
 
   ADD_TEST ("/cogl/shaders", test_cogl_just_vertex_shader);
   ADD_TEST ("/cogl/shaders", test_cogl_pipeline_uniforms);
@@ -167,10 +169,11 @@ main (int argc, char **argv)
 
   ADD_TEST ("/cogl/internal/bitmask", test_cogl_bitmask);
 
+  ADD_TEST ("/cogl", test_cogl_offscreen);
+
   /* left to the end because they aren't currently very orthogonal and tend to
    * break subsequent tests! */
   UNPORTED_TEST ("/cogl", test_cogl_viewport);
-  UNPORTED_TEST ("/cogl", test_cogl_offscreen);
 
   return g_test_run ();
 }
