@@ -6,11 +6,14 @@
 
 typedef struct _TestState
 {
-  int stub;
+  CoglContext *ctx;
+  CoglFramebuffer *fb;
 } TestState;
 
+typedef void (* SnippetTestFunc) (TestState *state);
+
 static CoglPipeline *
-create_texture_pipeline (void)
+create_texture_pipeline (TestState *state)
 {
   CoglPipeline *pipeline;
   CoglHandle tex;
@@ -27,7 +30,7 @@ create_texture_pipeline (void)
                                     8, /* rowstride */
                                     tex_data);
 
-  pipeline = cogl_pipeline_new ();
+  pipeline = cogl_pipeline_new (state->ctx);
 
   cogl_pipeline_set_layer_texture (pipeline, 0, tex);
 
@@ -41,22 +44,13 @@ create_texture_pipeline (void)
 }
 
 static void
-paint (TestState *state)
+simple_fragment_snippet (TestState *state)
 {
   CoglPipeline *pipeline;
   CoglSnippet *snippet;
-  CoglMatrix matrix, identity_matrix;
-  CoglColor color;
-  int location;
-  int i;
-
-  cogl_matrix_init_identity (&identity_matrix);
-
-  cogl_color_init_from_4ub (&color, 0, 0, 0, 255);
-  cogl_clear (&color, COGL_BUFFER_BIT_COLOR);
 
   /* Simple fragment snippet */
-  pipeline = cogl_pipeline_new ();
+  pipeline = cogl_pipeline_new (state->ctx);
 
   cogl_pipeline_set_color4ub (pipeline, 255, 0, 0, 255);
 
@@ -72,8 +66,17 @@ paint (TestState *state)
 
   cogl_object_unref (pipeline);
 
+  test_utils_check_pixel (5, 5, 0xffff00ff);
+}
+
+static void
+simple_vertex_snippet (TestState *state)
+{
+  CoglPipeline *pipeline;
+  CoglSnippet *snippet;
+
   /* Simple vertex snippet */
-  pipeline = cogl_pipeline_new ();
+  pipeline = cogl_pipeline_new (state->ctx);
 
   cogl_pipeline_set_color4ub (pipeline, 255, 0, 0, 255);
 
@@ -89,9 +92,19 @@ paint (TestState *state)
 
   cogl_object_unref (pipeline);
 
+  test_utils_check_pixel (15, 5, 0xff00ffff);
+}
+
+static void
+shared_uniform (TestState *state)
+{
+  CoglPipeline *pipeline;
+  CoglSnippet *snippet;
+  int location;
+
   /* Snippets sharing a uniform across the vertex and fragment
      hooks */
-  pipeline = cogl_pipeline_new ();
+  pipeline = cogl_pipeline_new (state->ctx);
 
   location = cogl_pipeline_get_uniform_location (pipeline, "a_value");
   cogl_pipeline_set_uniform_1f (pipeline, location, 0.25f);
@@ -115,8 +128,19 @@ paint (TestState *state)
 
   cogl_object_unref (pipeline);
 
+  test_utils_check_pixel (25, 5, 0xff0080ff);
+}
+
+static void
+lots_snippets (TestState *state)
+{
+  CoglPipeline *pipeline;
+  CoglSnippet *snippet;
+  int location;
+  int i;
+
   /* Lots of snippets on one pipeline */
-  pipeline = cogl_pipeline_new ();
+  pipeline = cogl_pipeline_new (state->ctx);
 
   cogl_pipeline_set_color4ub (pipeline, 0, 0, 0, 255);
 
@@ -150,9 +174,18 @@ paint (TestState *state)
 
   cogl_object_unref (pipeline);
 
+  test_utils_check_pixel (35, 5, 0x19334cff);
+}
+
+static void
+shared_variable_pre_post (TestState *state)
+{
+  CoglPipeline *pipeline;
+  CoglSnippet *snippet;
+
   /* Test that the pre string can declare variables used by the post
      string */
-  pipeline = cogl_pipeline_new ();
+  pipeline = cogl_pipeline_new (state->ctx);
 
   cogl_pipeline_set_color4ub (pipeline, 255, 255, 255, 255);
 
@@ -169,6 +202,15 @@ paint (TestState *state)
 
   cogl_object_unref (pipeline);
 
+  test_utils_check_pixel (45, 5, 0xff0000ff);
+}
+
+static void
+test_pipeline_caching (TestState *state)
+{
+  CoglPipeline *pipeline;
+  CoglSnippet *snippet;
+
   /* Check that the pipeline caching works when unrelated pipelines
      share snippets state. It's too hard to actually assert this in
      the conformance test but at least it should be possible to see by
@@ -181,14 +223,14 @@ paint (TestState *state)
                               "   unrelated pipelines */",
                               "cogl_color_out = vec4 (0.0, 1.0, 0.0, 1.0);\n");
 
-  pipeline = cogl_pipeline_new ();
+  pipeline = cogl_pipeline_new (state->ctx);
   cogl_pipeline_add_snippet (pipeline, snippet);
   cogl_push_source (pipeline);
   cogl_rectangle (50, 0, 60, 10);
   cogl_pop_source ();
   cogl_object_unref (pipeline);
 
-  pipeline = cogl_pipeline_new ();
+  pipeline = cogl_pipeline_new (state->ctx);
   cogl_pipeline_add_snippet (pipeline, snippet);
   cogl_push_source (pipeline);
   cogl_rectangle (60, 0, 70, 10);
@@ -197,6 +239,16 @@ paint (TestState *state)
 
   cogl_object_unref (snippet);
 
+  test_utils_check_pixel (55, 5, 0x00ff00ff);
+  test_utils_check_pixel (65, 5, 0x00ff00ff);
+}
+
+static void
+test_replace_string (TestState *state)
+{
+  CoglPipeline *pipeline;
+  CoglSnippet *snippet;
+
   /* Check the replace string */
   snippet = cogl_snippet_new (COGL_SNIPPET_HOOK_FRAGMENT, NULL, NULL);
   cogl_snippet_set_pre (snippet,
@@ -208,7 +260,7 @@ paint (TestState *state)
   cogl_snippet_set_post (snippet,
                          "cogl_color_out += vec4 (0.5, 0.0, 0.0, 1.0);");
 
-  pipeline = cogl_pipeline_new ();
+  pipeline = cogl_pipeline_new (state->ctx);
   cogl_pipeline_add_snippet (pipeline, snippet);
   cogl_push_source (pipeline);
   cogl_rectangle (70, 0, 80, 10);
@@ -217,6 +269,15 @@ paint (TestState *state)
 
   cogl_object_unref (snippet);
 
+  test_utils_check_pixel (75, 5, 0x808000ff);
+}
+
+static void
+test_texture_lookup_hook (TestState *state)
+{
+  CoglPipeline *pipeline;
+  CoglSnippet *snippet;
+
   /* Check the texture lookup hook */
   snippet = cogl_snippet_new (COGL_SNIPPET_HOOK_TEXTURE_LOOKUP,
                               NULL,
@@ -225,7 +286,7 @@ paint (TestState *state)
      get the green texel */
   cogl_snippet_set_pre (snippet, "cogl_tex_coord.x = 1.0 - cogl_tex_coord.x;");
 
-  pipeline = create_texture_pipeline ();
+  pipeline = create_texture_pipeline (state);
   cogl_pipeline_add_layer_snippet (pipeline, 0, snippet);
   cogl_push_source (pipeline);
   cogl_rectangle_with_texture_coords (80, 0, 90, 10,
@@ -235,11 +296,48 @@ paint (TestState *state)
 
   cogl_object_unref (snippet);
 
+  test_utils_check_pixel (85, 5, 0x00ffffff);
+}
+
+static void
+test_multiple_samples (TestState *state)
+{
+  CoglPipeline *pipeline;
+  CoglSnippet *snippet;
+
+  /* Check that we can use the passed in sampler in the texture lookup
+     to sample multiple times */
+  snippet = cogl_snippet_new (COGL_SNIPPET_HOOK_TEXTURE_LOOKUP,
+                              NULL,
+                              NULL);
+  cogl_snippet_set_replace (snippet,
+                            "cogl_texel = "
+                            "texture2D (cogl_sampler, vec2 (0.25, 0.25)) + "
+                            "texture2D (cogl_sampler, vec2 (0.75, 0.25));");
+
+  pipeline = create_texture_pipeline (state);
+  cogl_pipeline_add_layer_snippet (pipeline, 0, snippet);
+  cogl_push_source (pipeline);
+  cogl_rectangle (0, 0, 10, 10);
+  cogl_pop_source ();
+  cogl_object_unref (pipeline);
+
+  cogl_object_unref (snippet);
+
+  test_utils_check_pixel (5, 5, 0xffff00ff);
+}
+
+static void
+test_replace_lookup_hook (TestState *state)
+{
+  CoglPipeline *pipeline;
+  CoglSnippet *snippet;
+
   /* Check replacing the texture lookup hook */
   snippet = cogl_snippet_new (COGL_SNIPPET_HOOK_TEXTURE_LOOKUP, NULL, NULL);
   cogl_snippet_set_replace (snippet, "cogl_texel = vec4 (0.0, 0.0, 1.0, 0.0);");
 
-  pipeline = create_texture_pipeline ();
+  pipeline = create_texture_pipeline (state);
   cogl_pipeline_add_layer_snippet (pipeline, 0, snippet);
   cogl_push_source (pipeline);
   cogl_rectangle_with_texture_coords (90, 0, 100, 10,
@@ -249,8 +347,17 @@ paint (TestState *state)
 
   cogl_object_unref (snippet);
 
+  test_utils_check_pixel (95, 5, 0x0000ffff);
+}
+
+static void
+test_replace_snippet (TestState *state)
+{
+  CoglPipeline *pipeline;
+  CoglSnippet *snippet;
+
   /* Test replacing a previous snippet */
-  pipeline = create_texture_pipeline ();
+  pipeline = create_texture_pipeline (state);
 
   snippet = cogl_snippet_new (COGL_SNIPPET_HOOK_FRAGMENT,
                               NULL,
@@ -271,8 +378,17 @@ paint (TestState *state)
   cogl_pop_source ();
   cogl_object_unref (pipeline);
 
+  test_utils_check_pixel (105, 5, 0xff0000ff);
+}
+
+static void
+test_replace_fragment_layer (TestState *state)
+{
+  CoglPipeline *pipeline;
+  CoglSnippet *snippet;
+
   /* Test replacing the fragment layer code */
-  pipeline = create_texture_pipeline ();
+  pipeline = create_texture_pipeline (state);
 
   snippet = cogl_snippet_new (COGL_SNIPPET_HOOK_LAYER_FRAGMENT, NULL, NULL);
   cogl_snippet_set_replace (snippet, "cogl_layer = vec4 (0.0, 0.0, 1.0, 1.0);");
@@ -294,8 +410,17 @@ paint (TestState *state)
   cogl_pop_source ();
   cogl_object_unref (pipeline);
 
+  test_utils_check_pixel (115, 5, 0xff00ffff);
+}
+
+static void
+test_modify_fragment_layer (TestState *state)
+{
+  CoglPipeline *pipeline;
+  CoglSnippet *snippet;
+
   /* Test modifying the fragment layer code */
-  pipeline = cogl_pipeline_new ();
+  pipeline = cogl_pipeline_new (state->ctx);
 
   cogl_pipeline_set_uniform_1f (pipeline,
                                 cogl_pipeline_get_uniform_location (pipeline,
@@ -314,8 +439,18 @@ paint (TestState *state)
   cogl_pop_source ();
   cogl_object_unref (pipeline);
 
+  test_utils_check_pixel (125, 5, 0xff80ffff);
+}
+
+static void
+test_modify_vertex_layer (TestState *state)
+{
+  CoglPipeline *pipeline;
+  CoglSnippet *snippet;
+  CoglMatrix matrix;
+
   /* Test modifying the vertex layer code */
-  pipeline = create_texture_pipeline ();
+  pipeline = create_texture_pipeline (state);
 
   cogl_matrix_init_identity (&matrix);
   cogl_matrix_translate (&matrix, 0.0f, 1.0f, 0.0f);
@@ -333,8 +468,18 @@ paint (TestState *state)
   cogl_pop_source ();
   cogl_object_unref (pipeline);
 
+  test_utils_check_pixel (135, 5, 0xffff00ff);
+}
+
+static void
+test_replace_vertex_layer (TestState *state)
+{
+  CoglPipeline *pipeline;
+  CoglSnippet *snippet;
+  CoglMatrix matrix;
+
   /* Test replacing the vertex layer code */
-  pipeline = create_texture_pipeline ();
+  pipeline = create_texture_pipeline (state);
 
   cogl_matrix_init_identity (&matrix);
   cogl_matrix_translate (&matrix, 0.0f, 1.0f, 0.0f);
@@ -353,8 +498,23 @@ paint (TestState *state)
   cogl_pop_source ();
   cogl_object_unref (pipeline);
 
+  test_utils_check_pixel (145, 5, 0x00ff00ff);
+}
+
+static void
+test_vertex_transform_hook (TestState *state)
+{
+  CoglPipeline *pipeline;
+  CoglSnippet *snippet;
+  CoglMatrix identity_matrix;
+  CoglMatrix matrix;
+  int location;
+
   /* Test the vertex transform hook */
-  pipeline = cogl_pipeline_new ();
+
+  cogl_matrix_init_identity (&identity_matrix);
+
+  pipeline = cogl_pipeline_new (state->ctx);
 
   cogl_pipeline_set_color4ub (pipeline, 255, 0, 255, 255);
 
@@ -388,6 +548,15 @@ paint (TestState *state)
   /* Restore the projection matrix */
   cogl_set_projection_matrix (&matrix);
 
+  test_utils_check_pixel (155, 5, 0xff00ffff);
+}
+
+static void
+test_snippet_order (TestState *state)
+{
+  CoglPipeline *pipeline;
+  CoglSnippet *snippet;
+
   /* Verify that the snippets are executed in the right order. We'll
      replace the r component of the color in the pre sections of the
      snippets and the g component in the post. The pre sections should
@@ -395,7 +564,7 @@ paint (TestState *state)
      sections in the same order as they were added. Therefore the r
      component should be taken from the the second snippet and the g
      component from the first */
-  pipeline = cogl_pipeline_new ();
+  pipeline = cogl_pipeline_new (state->ctx);
 
   cogl_pipeline_set_color4ub (pipeline, 0, 0, 0, 255);
 
@@ -419,6 +588,54 @@ paint (TestState *state)
   cogl_pop_source ();
   cogl_object_unref (pipeline);
 
+  test_utils_check_pixel (165, 5, 0x80ff00ff);
+}
+
+static void
+test_naming_texture_units (TestState *state)
+{
+  CoglPipeline *pipeline;
+  CoglSnippet *snippet;
+  CoglTexture *tex1, *tex2;
+
+  /* Test that we can sample from an arbitrary texture unit by naming
+     its layer number */
+
+  snippet = cogl_snippet_new (COGL_SNIPPET_HOOK_FRAGMENT,
+                              NULL,
+                              NULL);
+  cogl_snippet_set_replace (snippet,
+                            "cogl_color_out = "
+                            "texture2D (cogl_sampler100, vec2 (0.0, 0.0)) + "
+                            "texture2D (cogl_sampler200, vec2 (0.0, 0.0));");
+
+  tex1 = test_utils_create_color_texture (state->ctx, 0xff0000ff);
+  tex2 = test_utils_create_color_texture (state->ctx, 0x00ff00ff);
+
+  pipeline = cogl_pipeline_new (state->ctx);
+
+  cogl_pipeline_set_layer_texture (pipeline, 100, tex1);
+  cogl_pipeline_set_layer_texture (pipeline, 200, tex2);
+
+  cogl_pipeline_add_snippet (pipeline, snippet);
+
+  cogl_push_source (pipeline);
+  cogl_rectangle (0, 0, 10, 10);
+  cogl_pop_source ();
+
+  cogl_object_unref (pipeline);
+  cogl_object_unref (snippet);
+  cogl_object_unref (tex1);
+  cogl_object_unref (tex2);
+
+  test_utils_check_pixel (5, 5, 0xffff00ff);
+}
+
+static void
+test_snippet_properties (TestState *state)
+{
+  CoglSnippet *snippet;
+
   /* Sanity check modifying the snippet */
   snippet = cogl_snippet_new (COGL_SNIPPET_HOOK_FRAGMENT, "foo", "bar");
   g_assert_cmpstr (cogl_snippet_get_declarations (snippet), ==, "foo");
@@ -455,26 +672,43 @@ paint (TestState *state)
                    COGL_SNIPPET_HOOK_FRAGMENT);
 }
 
+static SnippetTestFunc
+tests[] =
+  {
+    simple_fragment_snippet,
+    simple_vertex_snippet,
+    shared_uniform,
+    lots_snippets,
+    shared_variable_pre_post,
+    test_pipeline_caching,
+    test_replace_string,
+    test_texture_lookup_hook,
+    test_multiple_samples,
+    test_replace_lookup_hook,
+    test_replace_snippet,
+    test_replace_fragment_layer,
+    test_modify_fragment_layer,
+    test_modify_vertex_layer,
+    test_replace_vertex_layer,
+    test_vertex_transform_hook,
+    test_snippet_order,
+    test_naming_texture_units,
+    test_snippet_properties
+  };
+
 static void
-validate_result (void)
+run_tests (TestState *state)
 {
-  test_utils_check_pixel (5, 5, 0xffff00ff);
-  test_utils_check_pixel (15, 5, 0xff00ffff);
-  test_utils_check_pixel (25, 5, 0xff0080ff);
-  test_utils_check_pixel (35, 5, 0x19334cff);
-  test_utils_check_pixel (45, 5, 0xff0000ff);
-  test_utils_check_pixel (55, 5, 0x00ff00ff);
-  test_utils_check_pixel (65, 5, 0x00ff00ff);
-  test_utils_check_pixel (75, 5, 0x808000ff);
-  test_utils_check_pixel (85, 5, 0x00ffffff);
-  test_utils_check_pixel (95, 5, 0x0000ffff);
-  test_utils_check_pixel (105, 5, 0xff0000ff);
-  test_utils_check_pixel (115, 5, 0xff00ffff);
-  test_utils_check_pixel (125, 5, 0xff80ffff);
-  test_utils_check_pixel (135, 5, 0xffff00ff);
-  test_utils_check_pixel (145, 5, 0x00ff00ff);
-  test_utils_check_pixel (155, 5, 0xff00ffff);
-  test_utils_check_pixel (165, 5, 0x80ff00ff);
+  int i;
+
+  for (i = 0; i < G_N_ELEMENTS (tests); i++)
+    {
+      cogl_framebuffer_clear4f (state->fb,
+                                COGL_BUFFER_BIT_COLOR,
+                                0, 0, 0, 1);
+
+      tests[i] (state);
+    }
 }
 
 void
@@ -488,6 +722,9 @@ test_cogl_snippets (TestUtilsGTestFixture *fixture,
     {
       TestState state;
 
+      state.ctx = shared_state->ctx;
+      state.fb = shared_state->fb;
+
       cogl_ortho (/* left, right */
                   0, cogl_framebuffer_get_width (shared_state->fb),
                   /* bottom, top */
@@ -495,8 +732,7 @@ test_cogl_snippets (TestUtilsGTestFixture *fixture,
                   /* z near, far */
                   -1, 100);
 
-      paint (&state);
-      validate_result ();
+      run_tests (&state);
 
       if (g_test_verbose ())
         g_print ("OK\n");
