@@ -85,8 +85,8 @@ compare_component (int a, int b)
   return ABS (a - b) <= 1;
 }
 
-static void
-compare_pixel (const guint8 *screen_pixel, guint32 expected_pixel)
+void
+test_utils_compare_pixel (const guint8 *screen_pixel, guint32 expected_pixel)
 {
   /* Compare each component with a small fuzz factor */
   if (!compare_component (screen_pixel[0], expected_pixel >> 24) ||
@@ -115,7 +115,7 @@ test_utils_check_pixel (int x, int y, guint32 expected_pixel)
                     COGL_PIXEL_FORMAT_RGBA_8888_PRE,
                     pixel);
 
-  compare_pixel (pixel, expected_pixel);
+  test_utils_compare_pixel (pixel, expected_pixel);
 }
 
 void
@@ -144,11 +144,28 @@ test_utils_check_region (int x, int y,
   for (y = 0; y < height; y++)
     for (x = 0; x < width; x++)
       {
-        compare_pixel (p, expected_rgba);
+        test_utils_compare_pixel (p, expected_rgba);
         p += 4;
       }
 
   g_free (pixels);
 }
 
+CoglTexture *
+test_utils_create_color_texture (CoglContext *context,
+                                 guint32 color)
+{
+  CoglTexture2D *tex_2d;
 
+  color = GUINT32_TO_BE (color);
+
+  tex_2d = cogl_texture_2d_new_from_data (context,
+                                          1, 1, /* width/height */
+                                          COGL_PIXEL_FORMAT_RGBA_8888_PRE,
+                                          COGL_PIXEL_FORMAT_RGBA_8888_PRE,
+                                          4, /* rowstride */
+                                          (guint8 *) &color,
+                                          NULL);
+
+  return COGL_TEXTURE (tex_2d);
+}
